@@ -86,33 +86,45 @@ from datafold.utils.general import is_integer
 
 
 class EDMD(Pipeline, TSCPredictMixIn):
-    """Extended Dynamic Mode Decomposition model (EDMD) to approximate the Koopman
-    operator to model a dynamical system from time series data with a dictionary of
-    observable functions.
+    """Extended Dynamic Mode Decomposition (EDMD) model to approximate the Koopman
+    operator with a matrix.
+
+    The model is trained with time series collection data to identify a dynamical system.
+    The implementation is a special case of :class:`sklearn.pipeline.Pipeline`,
+    where the EDMD dictionary contains a flexible number of transform models and the
+    final estimator is a :class:`.DMDBase` model, which decomposes the time series data
+    into spatio-temporal components.
+
+    * *original* data refers to the original time series measurements (used to `fit`
+      the model)
+    * *dictionary* data refers to the data after it was transformed by the dictionary
+      and before it is processed by the DMD model
 
     ...
 
     Parameters
     ----------
     dict_steps
-        The order in the list determines the order at which the object's `fit/transform`
-        (and inverse_transform if necessary, see ``include_id_state``)) are processed.
-        All objects in the tuple must be able to deal with time series collection data.
+        List with `(string_identifier, model)` of models to transform the data. The
+        list defines the transformation pipeline and order of execution. All models in
+        the list must be able to handle :class:`.TSCDataFrame` as input and output.
 
     dmd_model
-        The final estimator to predict time series (in the dictionary space).
-        Approximates the Koopman matrix in observable space.
+        A DMD variant as the The final estimator to approximate the Koopman operator
+        with a matrix. The model predicts time series in the dictionary space that then
+        need to be transformed to the original space.
 
     include_id_state
-        If True the original time series states are added to the dictionary (without
-        any transformations). After the time series are created in the observable
-        space, the mapping back to the original space is easier (at the cost of a
-        larger dictionary) and allows for transformer in the dictionary that do not
-        implement an ``inverse_transform``.
+        If True, the original time series samples are added to the dictionary (without
+        any transformations), after the dictionary space time series are created. The
+        mapping from the dictionary space time series back to the original space
+        becomes much easier, at the cost of a larger dictionary dimension. If enabled,
+        also the dictionary can include models that do not provide an
+        ``inverse_transform`` function.
 
         .. note::
-            The final dictionary TSCDataFrame must not contain any feature names of the
-            original features.
+            The final dictionary :class:`.TSCDataFrame` must not contain any feature names
+            of the original feature names to avoid duplicates.
 
     memory: :class:`Optional[None, str, object]`, :class:`object` with the \
     `joblib.Memory` interface
@@ -182,7 +194,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
                 )
 
     def transform(self, X: TransformType) -> TransformType:
-        """Perform dictionary transformations on time series data in original space.
+        """Perform dictionary transformations on time series data (original space).
 
         Parameters
         ----------
@@ -213,7 +225,8 @@ class EDMD(Pipeline, TSCPredictMixIn):
         return X_dict
 
     def inverse_transform(self, X: TransformType) -> TransformType:
-        """Map time series from the dictionary space back to the original space.
+        """Perform inverse dictionary transformations on time series data (dictionary
+        space).
 
         * ``include_id_state=True`` - simply select the original features from the time \
           series.
@@ -302,19 +315,19 @@ class EDMD(Pipeline, TSCPredictMixIn):
         return X
 
     def fit(self, X: TimePredictType, y=None, **fit_params) -> "EDMD":
-        r"""Train EDMD model.
+        r"""Fit the EDMD model.
 
-        Fit all the transforms one after the other and transform the
-        data, then fit the transformed data using the DMD model.
+        Internally calls `fit_transform` of all transform methods in the dictionary (in
+        same order) and then fits the DMD model (final estimator).
 
         Parameters
         ----------
         X
-            Training data. Must fulfill input requirements of first step of the
-            pipeline.
+            Training time series data. Must fulfill input requirements of first
+            `dict_step` in the dictionary pipeline.
 
         y : None
-            Ignored parameter.
+            ignored
 
         **fit_params: Dict[str, object]
             Parameters passed to the ``fit`` method of each step, where
@@ -329,7 +342,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
         Raises
         ------
         TSCException
-            Time series collection restrictions in **X**: (1) time delta must be constant
+            Time series collection restrictions in `X`: (1) time delta must be constant
             (2) all values must be finite (no `NaN` or `inf`)
         """
         self._validate_data(
@@ -359,43 +372,50 @@ class EDMD(Pipeline, TSCPredictMixIn):
         time_values: Optional[np.ndarray] = None,
         **predict_params,
     ):
-        """Perform dictionary transformations on initial condition, predict
-        time series with the final estimator (DMD model) for requested time values, and
-        map the time series states' back to original state.
+        """Time series predictions for one or many initial conditions.
+
+        The internal prediction steps:
+
+        1. Perform dictionary transformations on initial condition input (`X`).
+        2. Use the transformed initial conditions and predict the time series in
+           dictionary space with the DMD model.
+        3. Map the dictionary space time series back to the original space.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Initial conditions for prediction.
+            Initial conditions states for prediction. The input type depends on the
+            number of samples ``n_samples_ic_`` that are required for an initial
+            condition:
 
             * ``n_samples_ic_ = 1`` a :class:`DataFrame` or `ndarray` is sufficient (per \
-            row one initial condition)
+              row one initial condition)
             
             * ``n_samples_ic_ > 1`` a :py:class:`TSCDataFrame` is required with each \
-            initial condition being a time series of ``n_samples_ic_`` identical time \
-            values.
+              initial condition being a time series of ``n_samples_ic_`` identical time \
+              values.
 
-            Must fulfill input requirements of first step of the pipeline.
+            The input must fulfill the input requirements of first step of the pipeline.
 
-        time_values: array_like, optional
-            Time values to evaluate the dynamical system, for each initial condition.
-            Defaults to time values contained in training data.
+        time_values
+            Time values to evaluate the model for each initial condition.
+            Defaults to time values contained in the data available during ``fit``. The
+            values should be ascending and must be non-negative numeric values.
 
         **predict_params: Dict[str, object]
-            Parameters to the ``predict`` called at the end of all
-            transformations in the pipeline.
+            Keyword arguments handled to the ``predict`` method of the DMD model.
 
         Returns
         -------
         TSCDataFrame
-            Predicted time series, each evaluated at the same time values.
+            Predicted time series, each evaluated at the specified time values.
 
         Raises
         ------
         TSCException
-            Time series collection restrictions in **X**: (1) same (constant) time
-            delta as during fit (2) all time series must have identical time values
-            (3) all values must be finite (no `NaN` or `inf`)
+            Time series collection requirements in `X`: (1) same (constant) time delta as
+            during fit (2) all time series must have identical
+            time values (3) all values must be finite (no `NaN` or `inf`)
         """
 
         check_is_fitted(self)
@@ -436,8 +456,12 @@ class EDMD(Pipeline, TSCPredictMixIn):
     def reconstruct(self, X: TSCDataFrame) -> TSCDataFrame:
         """Reconstruct existing time series collection.
 
-        Extract the same initial states from the time series in the collection and
-        predict the other states with the model at the same time values.
+        Internal steps to reconstruct a time series collection:
+
+        1. Extract the initial states from each time series in the collection (
+           this can also be multiple time samples, see attribute ``n_samples_ic_``).
+        2. Predict the remaining states of each time series with the EDMD model at the
+           same time values.
 
         Parameters
         ----------
@@ -453,7 +477,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
         Raises
         ------
         TSCException
-            Time series collection restrictions in `X`: (1) time delta must be constant
+            Time series collection requirements in `X`: (1) time delta must be constant
             (2) all values must be finite (no `NaN` or `inf`)
         """
         check_is_fitted(self)
@@ -494,7 +518,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
         Parameters
         ----------
         X
-            Training time series. Must fulfill input requirements of first step of
+            Training time series data. Must fulfill input requirements of first step of
             the pipeline.
 
         y: None
@@ -520,14 +544,12 @@ class EDMD(Pipeline, TSCPredictMixIn):
         return self.fit(X=X, y=y, **fit_params).reconstruct(X=X)
 
     def fit_transform(self, X: TSCDataFrame, y=None, **fit_params):
-        """
-        Fit the dictionary and the DMD model and return the transformed data.
+        """Fit the dictionary and the DMD model and return the transformed time series.
 
         Parameters
         ----------
         X
-            Time series collection data to fit the model with and return the
-            dictionary transformed data.
+            Time series collection data to fit the model.
 
         y: None
             ignored
@@ -540,8 +562,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
         Returns
         -------
         TSCDataFrame
-            `X_transformed` - Time series collection after applying all
-            dictionary transformations.
+             Transformed time series data in dictionary space.
 
         Raises
         ------
@@ -555,20 +576,21 @@ class EDMD(Pipeline, TSCPredictMixIn):
     def score(
         self, X: TSCDataFrame, y=None, sample_weight: Optional[np.ndarray] = None
     ):
-        """Apply transforms, and score with the final estimator
+        """Reconstruct the time series collection and score the recunstructed time
+        series with the original time series.
 
         Parameters
         ----------
         X
             Time series collection to reconstruct and score. Must fulfill input
-            requirements of all disctionary transformations.
+            requirements of all dictionary models.
 
         y: None
             ignored
 
         sample_weight
             If not None, this argument is passed as ``sample_weight`` keyword
-            argument to the ``score`` method set up.
+            argument to the internal ``score`` method.
 
         Returns
         -------
@@ -578,7 +600,7 @@ class EDMD(Pipeline, TSCPredictMixIn):
         Raises
         ------
         TSCException
-            Time series collection restrictions in **X**: (1) time delta must be constant
+            Time series collection restrictions in `X`: (1) time delta must be constant
             (2) all values must be finite (no `NaN` or `inf`)
         """
 
@@ -732,17 +754,15 @@ def _fit_and_score_edmd(
 
 
 class EDMDCV(GridSearchCV, TSCPredictMixIn):
-    """Exhaustive search over specified parameter values for an :class:`EDMD` model.
-
-    The parameters of the estimator used to apply these methods are optimized
-    by cross-validated grid-search over a parameter grid.
+    """Exhaustive parameter search over specified grid for a :class:`EDMD` model with
+    cross-validation.
 
     ...
 
     Parameters
     ----------
     estimator
-        EDMD to carry out search the parameter space with. Either use the default ``score``
+        Model to be optimized. Either use the default ``score``
         function, or ``scoring`` must be passed.
 
     param_grid
@@ -752,7 +772,7 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
         in the list are explored. This enables searching over any sequence
         of parameter settings.
 
-    scoring : string, callable, list/tuple, dict or None, default: None
+    scoring : string, callable, list/tuple, dict or None
         A single string (see :ref:`scoring_parameter`) or a callable
         (see :ref:`scoring`) to evaluate the predictions on the test set.
 
@@ -767,6 +787,9 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
 
         If None, the estimator's score method is used.
 
+        .. warning::
+            The multi-metric optimization is experimental. Please use with care.
+
     n_jobs : int or None, optional (default=None)
         Number of jobs to run in parallel.
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
@@ -793,9 +816,9 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
     cv : cross-validation generator
         Determines the cross-validation splitting strategy. Possible inputs for cv are:
 
-        - :class:`TSCKfoldSeries` splits `k` folds across time series (useful when
-            many time series are in the data set)
-        - :class:`TSCKFoldTime` splits `k` folds across time
+        - :class:`.TSCKfoldSeries` splits `k` folds across time series (useful when
+            many time series are in a collection)
+        - :class:`.TSCKFoldTime` splits `k` folds across time
 
     refit : bool, string, or callable, default=True
         Refit an estimator using the best found parameters on the whole
@@ -835,8 +858,7 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
 
     return_train_score : :class:`bool`
         If ``False``, the ``cv_results_`` attribute will not include training
-        scores.
-        Computing training scores is used to get insights on how different
+        scores. Computing training scores is used to get insights on how different
         parameter settings impact the overfitting/underfitting trade-off.
         However computing the scores on the training set can be computationally
         expensive and is not strictly required to select the parameters that
@@ -847,7 +869,7 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
 
     cv_results_ : dict of numpy (masked) ndarrays
         A dict with keys as column headers and values as columns, that can be
-        imported into a pandas ``DataFrame``.
+        imported into a ``pandas.DataFrame``.
         For instance the below given table
 
         +------------+-----------+------------+-----------------+---+---------+
@@ -950,7 +972,6 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
     `pre_dispatch` many times. A reasonable value for `pre_dispatch` is `2 *
     n_jobs`.
 
-
     """
 
     def __init__(
@@ -1010,13 +1031,13 @@ class EDMDCV(GridSearchCV, TSCPredictMixIn):
         ----------
 
         X
-            Time series collection used for training.
+            Training time series data.
 
         y: None
             ignored
 
         **fit_params : Dict[str, object]
-            Parameters passed to the ``fit`` method of the estimator
+            Parameters passed to the ``fit`` method of the estimator.
         """
         self._validate_settings_edmd()
         X = self._validate_data(X)
