@@ -29,14 +29,29 @@ InitialConditionType = Union[TSCDataFrame, pd.DataFrame, np.ndarray]
 
 
 class TSCBaseMixIn:
+    """Base class to provide functionality required in the MixIn's provided in *datafold*.
+
+    See Also
+    --------
+
+    :class:`.TSCTransformerMixIn`
+
+    :class:`.TSCPredictMixIn`
+    """
+
     def _strictly_pandas_df(self, df):
-        # This returns False for subclasses (TSCDataFrame)
+        """Check if the type is strcitly a pandas.Dataframe (i.e., is False for
+        TSCDataFrame).
+        """
         return type(df) == pd.DataFrame
 
     def _has_feature_names(self, _obj):
+        # True, for pandas.DataFrame or TSCDataFrame
         return isinstance(_obj, pd.DataFrame)
 
     def _X_to_numpy(self, X):
+        """ Returns a numpy array of the data.
+        """
         if self._has_feature_names(X):
             X = X.to_numpy()
             # a row in a df is always a single sample (which requires to be
@@ -63,8 +78,8 @@ class TSCBaseMixIn:
         validate_array_kwargs=None,
         validate_tsc_kwargs=None,
     ):
-        """Provides a general function to check data -- can be overwritten if an
-        implementation requires different checks."""
+        """Provides a general function to validate data -- can be overwritten if a
+        concrete implementation requires different checks."""
 
         if validate_array_kwargs is None:
             validate_array_kwargs = {}
@@ -80,7 +95,8 @@ class TSCBaseMixIn:
 
         if not isinstance(X, TSCDataFrame):
             # Currently, a pd.DataFrame is treated like numpy data
-            #  -- there is no time required such as in TSCDataFrame
+            #  -- there is no assumption of the content in on the index (for
+            #  TSCDataFrame it is time series ID and time)
 
             validate_tsc_kwargs = {}  # no need to check
 
@@ -152,23 +168,26 @@ class TSCBaseMixIn:
 
 
 class TSCTransformerMixIn(TSCBaseMixIn, TransformerMixin):
-    """Provides infrastructure for (optional) time series transformations.
+    """Mixin to provide functionality for point cloud and time series transformations.
 
-    Allows to transform data with numpy.ndarray or pandas.DataFrame (including
-    TSCDataFrame). Usually, the input type is the output type.
+    Generally, the following input/output types are supported:
+
+    * :class:`numpy.ndarray`
+    * :class:`pandas.DataFrame` no restriction on the frame's index and column format
+    * :class:`.TSCDataFrame` as a special data frame for time series collections
 
     Parameters
     ----------
 
     features_in_: Tuple[int, pandas.Index]
-        Number of features during fit and corresponding feature names (intended to
-        be stored during `fit`). Suitable to validate intput data.
+        Number of features during fit and corresponding feature names. The attribute
+        should be set in during `fit`. Set feature names in `inverse_transform` and
+        validate input in `transform`.
 
     features_out_: Tuple[int, pandas.Index]
-        Number of features after trasnform and corresponding feature names (intended to
-        be stored during `fit`). Suitable to set feature names for transform and
-        validate input for `inverse_transform`.
-
+        Number of features and corresponding feature names after transformation.
+        The attribute should be set in during `fit`. Set feature names in
+        `transform` and validate input in `inverse_transform`.
     """
 
     _feature_attrs = ["features_in_", "features_out_"]
@@ -229,7 +248,7 @@ class TSCTransformerMixIn(TSCBaseMixIn, TransformerMixin):
 
         if not self._has_feature_names(X) or self.features_out_[1] is None:
             # Either
-            # * now X has no feature names, or
+            # * X has no feature names, or
             # * during fit X had no feature names given.
             # --> Only check if shape is correct and trust user with the rest
             if self.features_in_[0] != X.shape[1]:
@@ -239,7 +258,7 @@ class TSCTransformerMixIn(TSCBaseMixIn, TransformerMixin):
                 )
         else:  # self._has_feature_names(X)
             # Now X has features and during fit features were given. So now we can
-            # check if both feature names match:
+            # check if feature names of X match with data during fit:
 
             _check_features: Tuple[int, pd.Index]
 
@@ -260,8 +279,25 @@ class TSCTransformerMixIn(TSCBaseMixIn, TransformerMixin):
                 pdtest.assert_index_equal(right=_check_features[1], left=X.columns)
 
     def _same_type_X(
-        self, X: TransformType, values: np.ndarray, set_columns: pd.Index
+        self, X: TransformType, values: np.ndarray, feature_names: pd.Index
     ) -> TransformType:
+        """Chooses the same type for input as type of `X`.
+
+        Parameters
+        ----------
+        X
+            Object from which the type will be inferred.
+
+        values
+            Data to transform in the same format as `X`.
+
+        feature_names
+            Feature names in case `X` is a :class:`pandas.DataFrame`.
+
+        Returns
+        -------
+
+        """
 
         _type = type(X)
 
@@ -270,38 +306,65 @@ class TSCTransformerMixIn(TSCBaseMixIn, TransformerMixin):
             # check for the special case, then for the more general case.
 
             return TSCDataFrame.from_same_indices_as(
-                X, values=np.asarray(values), except_columns=set_columns
+                X, values=np.asarray(values), except_columns=feature_names
             )
         elif isinstance(X, pd.DataFrame):
-            return pd.DataFrame(values, index=X.index, columns=set_columns)
+            return pd.DataFrame(values, index=X.index, columns=feature_names)
         else:
             try:
-                # try to view as numpy.array
+                # last resort: try to view as numpy.array
                 values = np.asarray(values)
             except Exception:
                 raise TypeError(f"input type {type(X)} is not supported.")
             else:
                 return values
 
+    def fit_transform(self, X: TransformType, y=None, **fit_params) -> TransformType:
+        """Fit to data, then transform it.
+
+        Fits transformer to `X` and `y` (only if applicable) with optional parameters
+        `fit_params` and returns a transformed version of `X`.
+
+        Parameters
+        ----------
+        X
+            Training data to transform of shape `(n_samples, n_features)`.
+
+        y : None
+            ignored
+
+        **fit_params : dict
+            Additional fit parameters.
+
+        Returns
+        -------
+        numpy.ndarray, pandas.DataFrame, TSCDataFrame
+            Transformed array of shape `(n_samples, n_transformed_features)` and of same
+            type as input `X`.
+        """
+        # This is only to overwrite the datafold documentation from scikit-learns docs
+        return super(TSCTransformerMixIn, self).fit_transform(X=X, y=y, **fit_params)
+
 
 class TSCPredictMixIn(TSCBaseMixIn):
-    """Methods and functionality to provide time series prediction.
+    """Mixin to provide functionality for models that train on time series data.
 
     Parameters
     ----------
 
     features_in_: Tuple[int, pandas.Index]
-        Number of features during fit and corresponding feature names (intended to
-        be stored during `fit`). Suitable to test or create initial conditions.
+        Number of features during fit and corresponding feature names. The attribute
+        should be set during `fit` and used to validate during `predict`.
 
     time_values_in_: Tuple[int, numpy.ndarray]
-        Number of time values and array with all actual time values in at least on time
-        series (intended to be stored during `fit`). This allows to check for
-        time extrapolation.
+        Number of time values and array with all time values during `fit`. Note
+        because in a time seres collection not all time series share the same time
+        samples, a time value has to appear in at least one time series to be listed in
+        `time_values_in_`. The attribute should be set during `fit`, can be used for
+        default time values and allows reasonable prediction intervals to be validated.
 
     dt_
-        time delta during fit
-
+        Time sampling in the data during fit.
     """
 
     _cls_feature_attrs = ["features_in_", "time_values_in_", "dt_"]
@@ -394,7 +457,9 @@ class TSCPredictMixIn(TSCBaseMixIn):
         self._check_attributes_set_up(check_attributes=["features_in_"])
 
         try:
-            pdtest.assert_index_equal(right=self.features_in_[1], left=X.columns)
+            pdtest.assert_index_equal(
+                right=self.features_in_[1], left=X.columns, check_names=False
+            )
         except AssertionError as e:
             raise ValueError(e.args[0])
 
@@ -430,70 +495,3 @@ class TSCPredictMixIn(TSCBaseMixIn):
 
     def predict(self, X: InitialConditionType, time_values=None, **predict_params):
         raise NotImplementedError("method not implemented")
-
-
-class DmapKernelMethod(BaseEstimator):
-    def __init__(
-        self,
-        epsilon: float,
-        n_eigenpairs: int,
-        cut_off,
-        is_stochastic: bool,
-        alpha: float,
-        symmetrize_kernel,
-        dist_backend,
-        dist_backend_kwargs,
-    ):
-        self.epsilon = epsilon
-        self.n_eigenpairs = n_eigenpairs
-        self.cut_off = cut_off
-        self.is_stochastic = is_stochastic
-        self.alpha = alpha
-        self.symmetrize_kernel = symmetrize_kernel
-        self.dist_backend = dist_backend
-        self.dist_backend_kwargs = dist_backend_kwargs
-
-    @property
-    def kernel_(self) -> DmapKernelFixed:
-        if not hasattr(self, "_kernel"):
-            raise AttributeError(
-                f"Subclass {type(self)} is not properly implemented. "
-                f"No attribute '_kernel' in KernelMethod subclass. Please report bug."
-            )
-
-        return getattr(self, "_kernel")
-
-    def _unsymmetric_kernel_matrix(self, kernel_matrix, basis_change_matrix):
-        inv_basis_change_matrix = scipy.sparse.diags(
-            np.reciprocal(basis_change_matrix.data.ravel())
-        )
-
-        return inv_basis_change_matrix @ kernel_matrix @ inv_basis_change_matrix
-
-    def _solve_eigenproblem(
-        self, kernel_matrix, basis_change_matrix
-    ) -> Tuple[np.ndarray, np.ndarray]:
-
-        try:
-            eigvals, eigvect = compute_kernel_eigenpairs(
-                kernel_matrix=kernel_matrix,
-                n_eigenpairs=self.n_eigenpairs,
-                is_symmetric=self.kernel_.is_symmetric,
-                is_stochastic=self.is_stochastic,
-                backend="scipy",
-            )
-        except NumericalMathError:
-            # re-raise with more details for the DMAP
-            raise NumericalMathError(
-                "Eigenvalues have non-negligible imaginary part (larger than "
-                f"{1e2 * sys.float_info.epsilon}. First try to use "
-                f"parameter 'symmetrize_kernel=True' (improves numerical stability) and "
-                f"only if this is not working adjust epsilon."
-            )
-
-        if basis_change_matrix is not None:
-            eigvect = basis_change_matrix @ eigvect
-
-        eigvect /= np.linalg.norm(eigvect, axis=0)[np.newaxis, :]
-
-        return np.real(eigvals), np.real(eigvect)
