@@ -40,30 +40,29 @@ class LinearDynamicalSystem(object):
             \frac{d}{dt} x(t) = \mathcal{A} \cdot x(t)\\
             \mathcal{A} \in \mathbb{R}^{[m \times m]}
 
-    This system can also be written in terms of a discrete-time system
+    This continuous-system representation can also be written in terms of a discrete-time
+    system
 
     - discrete
         .. math::
             x_{n+1} = A \cdot x_{n}
 
-        and :math:`A = \exp(\mathcal{A} \Delta t)`, a constant matrix, which describes the
-        linear evolution of the systems' states :math:`x` with state length :math:`m`.
-
+    and :math:`A = \exp(\mathcal{A} \Delta t)`, a constant matrix, which describes the
+    linear evolution of the systems' states :math:`x` with state length :math:`m`.
 
     Parameters
     ----------
 
     mode
-        type of linear system
+        Type of linear system:
 
         * "continuous"
         * "discrete" (restricts time values to integer values)
 
     time_invariant
         If True, the system internally always starts with `time=0`. \
-        This is irrespective of the time given in the time values (If initial time is
-        larger than zero, the internal times are then corrected to the the requested
-        time).
+        This is irrespective of the time given in the time values. If the initial
+        time is larger than zero, the internal times are corrected to the requested time.
 
     References
     ----------
@@ -129,8 +128,8 @@ class LinearDynamicalSystem(object):
         time_series_ids: Optional[Dict] = None,
         feature_columns: Optional[Union[pd.Index, list]] = None,
     ):
-        r"""Evolve the dynamical system with the spectral components of the dynamical
-        matrix in the linear system.
+        r"""Evolve the dynamical system with spectral components of the dynamical
+        matrix.
 
         Using the eigenvalues :math:`\Lambda` and eigenvectors :math:`\Psi` of the
         constant matrix :math:`A`
@@ -149,64 +148,63 @@ class LinearDynamicalSystem(object):
             .. math::
                 x_{n+1} = \Psi \cdot \Lambda^n \cdot b_{0}
 
-        Where :math:`b(0)` and :math:`b_{0}` are the initial
+        where :math:`b(0)` and :math:`b_{0}` are the initial
         conditions of the respective system.
 
         .. note::
             Initial condition states :math:`x` of the original system need to be
             aligned to the right eigenvectors beforehand:
 
-            * By using the right eigenvetors and solve in a least square sense, or
+            * By using the right eigenvectors and solving in a least square sense
                 .. math::
                     \Psi_r x_0 = b_0
 
-            * by using the left eigenvectors and computing the matrix-vector product
+            * or by using the left eigenvectors and computing the matrix-vector product
                 .. math::
                     \Psi_l x_0 = b_0
 
         Parameters
         ----------
         dynmatrix
-            Spectral linear time map with shape `(n_feature, n_feature_states)`, \
+            Spectral linear time map of shape `(n_feature, n_feature_states)`, \
             where `n_feature_states` is the length of the initial condition.
 
             * right eigenvectors :math:`\Psi` of matrix :math:`A` (in this case \
               `n_feature=n_feature_states`), or
-            * linear transformation of right eigenvectors :math:`D \cdot \Psi` (this \
-              case allows `n_feature` to be larger or smaller than `n_feature_states`). \
-              The matrix :math:`D` allows to linearly map the states directly to another \
-              space (e.g. only a selection of states to reduce memory footprint).
+            * linear transformation of right eigenvectors :math:`D \cdot \Psi`. This \
+              allows `n_feature` to be larger or smaller than `n_feature_states`. \
+              The matrix :math:`D` maps the states directly to another \
+              space, e.g., only a selection of states for reduce memory footprint.
 
         eigenvalues
             eigenvalues of matrix :math:`A`
 
         time_delta
-            time delta :math:`\Delta t` of a continuous system
+            Time delta :math:`\Delta t` for a continuous system.
 
         initial_conditions
-            one initial condition with shape `(n_features,)` or multiple with shape \
-            `(n_features, n_initial_conditions)`
+            Single initial condition of shape `(n_features,)` or multiple of shape \
+            `(n_features, n_initial_conditions)`.
 
         time_values
-           Values to evaluate the linear system at
+           Time values to evaluate the linear system at
 
            * `mode="continuous"` - :math:`t \in \mathbb{R}^{+}`
            * `mode="discrete"` - :math:`n \in \mathbb{N}_0`
 
         time_series_ids
-           Unique integer time series IDs with shape `(n_initial_conditions,)` for each \
-           respective initial condition in the resulting time series collection. \
-           Defaults to (0, 1, 2, ...).
+           Unique integer time series IDs of shape `(n_initial_conditions,)` for each \
+           respective initial condition. Defaults to `(0, 1, 2, ...)`.
 
         feature_columns
-            Unique feature columns names with shape `(n_feature,)` in the result time \
-            series collection. Defaults to (0, 1, 2, ...).
+            Unique feature columns names of shape `(n_feature,)`.
+            Defaults to `(0, 1, 2, ...)`.
 
         Returns
         -------
         TSCDataFrame
-            collection with a time series for each initial condition with \
-            shape `(n_time_values, n_features)`
+            Collection with a time series for each initial condition with \
+            shape `(n_time_values, n_features)`.
         """
 
         n_feature, state_length = dynmatrix.shape
@@ -256,21 +254,29 @@ class LinearDynamicalSystem(object):
 
 
 class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
-    r"""Abstract base class for Dynamic Mode Decomposition (DMD) models to approximate
-    the Koopman operator.
+    r"""Abstract base class for Dynamic Mode Decomposition (DMD) models.
 
-    The Koopman matrix :math:`K` defines a linear dynamical system
+    A DMD model decomposes time series data linearly into spatio-temporal components.
+    Due to it's strong connection to non-linear dynamical systems with Koopman spectral
+    theory, the DMD variants (subclasses) are framed in the context of this theory.
 
-    .. math:: K^n x_0 &= x_k
+    A DMD model approximates the Koopman operator with a matrix :math:`K`,
+    which defines a linear dynamical system
 
-    with :math:`x_k` being the (column) state vector of the system at time :math:`k`.
-    When solving the eigenpairs of the Koopman matrix \
-    (:math:`\Psi_r` right eigenvectors, and :math:`\Lambda` eigenvalues on diagonal):
+    .. math:: K^n x_0 &= x_n
+
+    with :math:`x_n` being the (column) state vectors of the system at time :math:`n`.
+    Note, that the state vectors :math:`x` are often not the original observations of a
+    system but states from a functional coordinate basis that seeks to linearize the
+    dynamics (see reference for details).
+
+    The spectrum of the Koopman matrix \
+    (:math:`\Psi_r` right eigenvectors, and :math:`\Lambda` eigenvalues on diagonal)
 
     .. math:: K \Psi_r = \Psi_r \Lambda
 
-    then this enables further analysis and inexpensive evaluation of the above linear
-    dynamical system (matrix power of diagonal matrix :math:`\Lambda` instead of
+    enables further analysis about the system and inexpensive evaluation of the
+    Koopman system (matrix power of diagonal matrix :math:`\Lambda` instead of
     :math:`K`):
 
     .. math::
@@ -278,8 +284,8 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
         &= K^n \Psi_r b  \\
         &= \Psi_r \Lambda^n b
 
-    With the initial condition :math:`b`, which is aligned to the linear system using the
-    eigenvectors of the Koopman matrix:
+    The vector :math:`b` indicates the initial condition, which is aligned to the
+    linear system using the eigenvectors of the Koopman matrix:
 
     1. in a least squares sense with the right eigenvectors already computed in a least \
        square sense
@@ -294,10 +300,15 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
            \Psi_l x_0 = b
 
 
-    All subclasses must provide the (right) eigenpairs :math:`\left(\Lambda, \Psi_r\right)`,
-    in respective attributes :code:`eigenvalues_` and :code:`eigenvectors_right_`. If
-    the left eigenvectors (attribute :code:`eigenvectors_left_`) are available the
-    initial condition always solves with the second case for :math:`b`.
+    All subclasses of ``DMDBase`` must provide the (right) eigenpairs
+    :math:`\left(\Lambda, \Psi_r\right)`, in respective attributes
+    :code:`eigenvalues_` and :code:`eigenvectors_right_`. If the left eigenvectors
+    (in attribute :code:`eigenvectors_left_`) are available the initial condition always
+    solves with the second case for :math:`b`, because this is more efficient.
+
+    References
+    ----------
+    :cite:`kutz_dynamic_2016`
 
     See Also
     --------
@@ -423,12 +434,12 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
     def predict(
         self, X: InitialConditionType, time_values=None, **predict_params
     ) -> TSCDataFrame:
-        """Predict time series data for each initial condition for the time values.
+        """Predict time series data for each initial condition and time values.
 
         Parameters
         ----------
         X: pandas.DataFrame, numpy.ndarray
-            Initial conditions with shape `(n_initial_condition, n_features)`.
+            Initial conditions of shape `(n_initial_condition, n_features)`.
 
         time_values
             Time values to evaluate the model at.
@@ -436,18 +447,18 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
         Keyword Args
         ------------
 
-        post_map: np.ndarray
+        post_map: numpy.ndarray
             A matrix that is combined with the right eigenvectors. \
             :code:`post_map @ eigenvectors_right_`.
 
-        feature_columns:
-            If post_map is given and the state lengths changes, new feature names must \
-            be provided.
+        feature_columns: pandas.Index
+            If `post_map` is given with a changed state length, then new feature names
+            must be provided.
 
         Returns
         -------
         TSCDataFrame
-            time series predictions with shape `(n_time_values, n_features)` for each
+            time series predictions of shape `(n_time_values, n_features)` for each
             initial condition
         """
 
@@ -479,7 +490,7 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
         )
 
     def reconstruct(self, X: TSCDataFrame):
-        """Reconstruct existing time series collection.
+        """Reconstruct time series collection.
 
         Extract the same initial states from the time series in the collection and
         predict the other states with the model at the same time values.
@@ -491,7 +502,8 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
 
         Returns
         -------
-        X_reconstruct: TSCDataFrame of same shape as input.
+        TSCDataFrame
+            same shape as input `X`
         """
 
         check_is_fitted(self)
@@ -514,7 +526,7 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
         return X_reconstruct_ts
 
     def fit_predict(self, X: TSCDataFrame, **fit_params):
-        """Fit the model and reconstruct the time series training data.
+        """Fit model and reconstruct the time series data.
 
         Parameters
         ----------
@@ -529,21 +541,21 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
         return self.fit(X, **fit_params).reconstruct(X)
 
     def score(self, X: TSCDataFrame, y=None, sample_weight=None) -> float:
-        """Score DMD model by reconstructing time series data with same initial condition.
+        """Score model by reconstructing time series data.
 
-        The default metric (see :class:`TSCMetric` used is mode="feature", "metric=rmse"
+        The default metric (see :class:`.TSCMetric` used is mode="feature", "metric=rmse"
         and "min-max" scaling.
 
         Parameters
         ----------
         X
-            time series data to reconstruct with `(n_samples, n_features)`
+            Time series data to reconstruct with `(n_samples, n_features)`.
 
         y: None
             ignored
 
         sample_weight
-            handled to :py:meth:`TSCScoring.__call__`
+            passed to :py:meth:`TSCScoring.__call__`.
 
         Returns
         -------
@@ -559,10 +571,9 @@ class DMDBase(BaseEstimator, TSCPredictMixIn, metaclass=abc.ABCMeta):
 
 
 class DMDFull(DMDBase):
-    r"""Dynamic Mode Decomposition of time series data without any prior dimensionality
-    reduction.
+    r"""Full Dynamic Mode Decomposition of time series data.
 
-    The Koopman matrix is approximated with
+    The model approximates the Koopman matrix with
 
     .. math::
         K X &= X' \\
@@ -577,12 +588,13 @@ class DMDFull(DMDBase):
     ----------
 
     is_diagonalize
-        Whether to also compute the left eigenvectors. If True, then there is no
-        additional least squares solution required for evaluating the linear dynamical
-        system (see :class:`LinearDynamicalSystem`)
+        If True, also the left eigenvectors are computed. This is more efficient to
+        solve for initial conditions, because there is no least
+        squares computation required for evaluating the linear dynamical
+        system (see :class:`LinearDynamicalSystem`).
 
     rcond: Optional[float]
-        Parameter handled to `numpy.linalg.lstsq <https://docs.scipy.org/doc/numpy/reference/generated/numpy.linalg.lstsq.html>`_
+        Parameter handled to :class:`numpy.linalg.lstsq`.
 
     Attributes
     ----------
@@ -594,13 +606,11 @@ class DMDFull(DMDBase):
         All right eigenvectors of Koopman matrix.
 
     eigenvectors_left_: numpy.ndarray
-        Left eigenvectors of Koopman matrix if ``is_diagonalize=True``.
+        All left eigenvectors of Koopman matrix if ``is_diagonalize=True``.
 
     koopman_matrix_: numpy.ndarray
-        Koopman matrix obtained from least squares
-
-        .. note::
-            This is subject to get removed, the spectral parts are sufficient.
+        Koopman matrix obtained from least squares. Only stored if
+        `store_koopman_matrix=True` during fit.
 
     References
     ----------
@@ -630,7 +640,7 @@ class DMDFull(DMDBase):
 
         # It is more suitable to get the shift_start and end in row orientation as this
         # is closer to the normal least squares parameter definition
-        shift_start_transposed, shift_end_transposed = X.tsc.shift_matrices(
+        shift_start_transposed, shift_end_transposed = X.tsc.compute_shift_matrices(
             snapshot_orientation="row"
         )
 
@@ -697,17 +707,23 @@ class DMDFull(DMDBase):
         koopman_matrix = koopman_matrix.T
         return koopman_matrix
 
-    def fit(self, X: TimePredictType, y=None, **fit_params) -> "DMDFull":
-        """Build Koopman matrix from time series data.
+    def fit(
+        self, X: TimePredictType, y=None, store_koopman_matrix=False, **fit_params
+    ) -> "DMDFull":
+        """Build Koopman matrix and its spectral components from time series data.
 
         Parameters
         ----------
         X
-            Training data.
+            Training time series data.
 
         y: None
             ignored
 
+        store_koopman_matrix
+            If True, the model stores the Koopman matrix in attribute
+            ``koopman_matrix_``, otherwise only the spectral components are stored.
+
         Returns
         -------
         DMDFull
@@ -721,14 +737,17 @@ class DMDFull(DMDBase):
         )
         self._setup_features_and_time_fit(X=X)
 
-        self.koopman_matrix_ = self._compute_koopman_matrix(X)
+        koopman_matrix_ = self._compute_koopman_matrix(X)
         self.eigenvalues_, self.eigenvectors_right_ = sort_eigenpairs(
-            *np.linalg.eig(self.koopman_matrix_)
+            *np.linalg.eig(koopman_matrix_)
         )
 
         if self.is_diagonalize:
             self._diagonalize_left_eigenvectors()
 
+        if store_koopman_matrix:
+            self.koopman_matrix_ = koopman_matrix_
+
         return self
 
 
@@ -736,53 +755,54 @@ class DMDEco(DMDBase):
     r"""Dynamic Mode Decomposition of time series data with prior singular value
     decomposition (SVD).
 
-    The singular velue decomposition reduces the data and the Koopman operator is
+    The singular value decomposition (SVD) reduces the data and the Koopman operator is
     computed in this reduced space. This DMD model is particularly interesting for high
-    dimensional data (large number of features).
+    dimensional data (large number of features), for example, solutions of partial
+    differential equations (PDE) with a fine grid.
 
-    The procedure is as follows:
+    The procedure of ``DMDEco`` is as follows:
 
     1. Compute the singular value decomposition of the data and use the leading `k`
     singular values and corresponding vectors in :math:`U` and :math:`V`.
 
-    .. math::
-        X \approx U \Sigma V^*
+      .. math::
+          X \approx U \Sigma V^*
 
-    2. Compute the Koopman matrix on the SVD coordinates:
+    2. Compute the Koopman matrix in the SVD coordinates:
 
-    .. math::
-        K = U^T X' V \Sigma^{-1}
+      .. math::
+          K = U^T X' V \Sigma^{-1}
 
-    3. Compute the eigenpairs (in matrix form):
+    3. Compute all eigenpairs of Koopman matrix:
 
-    .. math::
-        K W_r = W_r \Omega
+      .. math::
+          K W_r = W_r \Omega
 
     4. Reconstruct the (exact) eigendecomposition of :math:`K`
 
-    .. math::
-        \Psi_r = X' V \Sigma^{-1} W
+      .. math::
+          \Psi_r = X' V \Sigma^{-1} W
 
-    .. note::
-        The eigenvectors in step 4 can also be computed with :math:`\Psi_r = U W`, which
-        is then referred to the projected reconstruction.
+      .. note::
+          The eigenvectors in step 4 can also be computed with :math:`\Psi_r = U W`, which
+          is then referred to the projected reconstruction.
 
     ...
 
     Parameters
     ----------
     svd_rank: int
-        Number of eigenpairs (with largest eigenvalues) to keep
-
+        Number of eigenpairs (with largest eigenvalues, in magnitude) to keep.
 
     Attributes
     ----------
 
     eigenvalues_ : numpy.ndarray
-        All eigenvalues (svd_rank,) of the (reduced) Koopman matrix .
+        All eigenvalues of shape `(svd_rank,)` of the (reduced) Koopman matrix .
 
     eigenvectors_right_ : numpy.ndarray
-        All right eigenvectors of the reduced Koopman matrix.
+        All right eigenvectors of shape `(svd_rank, svd_rank)` of the reduced Koopman
+        matrix.
 
     References
     ----------
@@ -802,7 +822,9 @@ class DMDEco(DMDBase):
         #  2 more quantities than snapshots
         #  Currently it is optimized for the case 2.
 
-        shift_start, shift_end = X.tsc.shift_matrices(snapshot_orientation="col")
+        shift_start, shift_end = X.tsc.compute_shift_matrices(
+            snapshot_orientation="col"
+        )
         U, S, Vh = np.linalg.svd(shift_start, full_matrices=False)  # (1.18)
 
         U = U[:, : self.svd_rank]
