@@ -12,16 +12,24 @@ from sklearn.utils.validation import check_is_fitted, check_scalar
 
 from datafold.decorators import warn_experimental_class
 from datafold.dynfold.base import DataFrameType, TransformType, TSCTransformerMixIn
-from datafold.pcfold import MultiquadricKernel, TSCDataFrame
+from datafold.pcfold import MultiquadricKernel, PCManifold, TSCDataFrame
 from datafold.pcfold.kernels import PCManifoldKernel
+from datafold.pcfold.timeseries.collection import TSCException
+
+try:
+    from findiff import FinDiff
+except ImportError:
+    IMPORTED_FINDIFF = False
+else:
+    IMPORTED_FINDIFF = True
 
 
 class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
-    """Apply a scikit-learn preprocess algorithms but generalize to time series
-    collections.
+    """Wrapper of a scikit-learn preprocess algorithms to allow time series
+    collections as input and output.
 
     Often scikit-learn performs "pandas.DataFrame in -> numpy.ndarray out". This wrapper
-    generalizes this to "pandas.DataFrame in -> pandas.DataFrame out".
+    makes sure to have "pandas.DataFrame in -> pandas.DataFrame out".
 
     Parameters
     ----------
@@ -61,12 +69,12 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
             )
 
     def fit(self, X: TransformType, y=None, **fit_params) -> "TSCFeaturePreprocess":
-        """Handle fit to internal transform sklearn object.
+        """Calls fit of internal transform ``sklearn`` object.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data with shape `(n_samples, n_features)`.
+            Training data of shape `(n_samples, n_features)`.
 
         y: None
             ignored
@@ -93,12 +101,12 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
         return self
 
     def transform(self, X: TransformType):
-        """Handle transform to internal sklearn object.
+        """Calls transform of internal transform ``sklearn`` object.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data to transform with shape `(n_samples, n_features)`.
+            Data to transform of shape `(n_samples, n_features)`.
 
         Returns
         -------
@@ -113,15 +121,17 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
 
         X_intern = self._X_to_numpy(X)
         values = self.sklearn_transformer_fit_.transform(X_intern)
-        return self._same_type_X(X=X, values=values, set_columns=self.features_out_[1])
+        return self._same_type_X(
+            X=X, values=values, feature_names=self.features_out_[1]
+        )
 
     def fit_transform(self, X: TransformType, y=None, **fit_params):
-        """Handle fit_transform to internal sklearn object.
+        """Calls fit_transform of internal transform ``sklearn`` object..
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data to transform with shape `(n_samples, n_features)`.
+            Training data to transform of shape `(n_samples, n_features)`.
 
         y: None
             ignored
@@ -139,15 +149,17 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
         self.sklearn_transformer_fit_ = clone(self.sklearn_transformer)
         values = self.sklearn_transformer_fit_.fit_transform(X)
 
-        return self._same_type_X(X=X, values=values, set_columns=self.features_out_[1])
+        return self._same_type_X(
+            X=X, values=values, feature_names=self.features_out_[1]
+        )
 
     def inverse_transform(self, X: TransformType):
-        """Handle inverse_transform to internal sklearn object.
+        """Calls `inverse_transform` of internal transform ``sklearn`` object.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data to map back with shape `(n_samples, n_features)`.
+            Data to map back of shape `(n_samples, n_features)`.
 
         Returns
         -------
@@ -162,11 +174,11 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixIn):
 
         X_intern = self._X_to_numpy(X)
         values = self.sklearn_transformer_fit_.inverse_transform(X_intern)
-        return self._same_type_X(X=X, values=values, set_columns=self.features_in_[1])
+        return self._same_type_X(X=X, values=values, feature_names=self.features_in_[1])
 
 
 class TSCIdentity(BaseEstimator, TSCTransformerMixIn):
-    """Dummy transformer for testing or  as a "passthrough" placeholder.
+    """Dummy transformer for testing or as a "passthrough" placeholder.
 
     Attributes
     ----------
@@ -183,7 +195,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data with shape `(n_samples, n_features)`.
+            Data of shape `(n_samples, n_features)`.
 
         y: None
             ignored
@@ -202,17 +214,17 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixIn):
         return self
 
     def transform(self, X: TransformType) -> TransformType:
-        """Passthrough data and validate features shape.
+        """Passthrough data and validate feature.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data with shape `(n_samples, n_features)` to passthrough.
+            Data of shape `(n_samples, n_features)` to passthrough.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type and hape as `X`
+            same type and shape as `X`
         """
         check_is_fitted(self, "is_fit_")
 
@@ -227,7 +239,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data to passthrough with shape `(n_samples, n_features)`.
+            Data to passthrough of shape `(n_samples, n_features)`.
 
         Returns
         -------
@@ -240,116 +252,14 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixIn):
         return X
 
 
-class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixIn):
-    def __init__(
-        self,
-        degree: int = 2,
-        interaction_only: bool = False,
-        include_bias: bool = False,
-        order: str = "C",
-    ):
-        super(TSCPolynomialFeatures, self).__init__(
-            degree=degree,
-            interaction_only=interaction_only,
-            include_bias=include_bias,
-            order=order,
-        )
-
-    def fit(self, X: TransformType, y=None, **fit_params):
-        X = self._validate_data(X)
-        # TODO: need to remove the ID state powers_ otherwise they are in twice... (or
-        #  make as option...)
-
-        super(TSCPolynomialFeatures, self).fit(self._X_to_numpy(X), y=y)
-
-        # Note1: the n_output_features_ is quite similar to TSCTransformerMixIn,
-        # but actually is set inside PolynomialFeatures
-        # Note3: the n_output_features_ are corrected by X.shape[1] because we can skip
-        self._setup_features_fit(
-            X, features_out=[f"poly{i}" for i in range(self.n_output_features_)],
-        )
-        return self
-
-    def transform(self, X: TransformType):
-        check_is_fitted(self)
-        X = self._validate_data(X)
-        self._validate_feature_input(X, direction="transform")
-
-        poly_data = super(TSCPolynomialFeatures, self).transform(self._X_to_numpy(X))
-        return self._same_type_X(X, values=poly_data, set_columns=self.features_out_[1])
-
-    def fit_transform(self, X, y=None, **fit_params):
-        X = self._validate_data(X)
-
-        pca_values = super(TSCPolynomialFeatures, self).fit_transform(
-            self._X_to_numpy(X), y=y
-        )
-
-        self._setup_features_fit(
-            X, features_out=[f"poly{i}" for i in range(self.n_output_features_)]
-        )
-
-        return self._same_type_X(
-            X, values=pca_values, set_columns=self.features_out_[1]
-        )
-
-
-class TSCApplyLambdas(BaseEstimator, TSCTransformerMixIn):
-    def __init__(self, lambda_list):
-        self.lambda_list = lambda_list
-
-    def fit(self, X: TransformType, y=None):
-
-        if isinstance(X, np.ndarray):
-            raise NotImplementedError(
-                "Currently not implemented for numpy.ndraay. If required please open "
-                "a Gitlab issue."
-            )
-
-        X = self._validate_data(X, ensure_feature_name_type=True)
-
-        features_out = [
-            f"{feature_name}_lambda{i}"
-            for feature_name in X.columns
-            for i in range(len(self.lambda_list))
-        ]
-
-        self._setup_features_fit(X, features_out=features_out)
-        return self
-
-    def transform(self, X: TransformType):
-        check_is_fitted(self)
-        X = self._validate_data(X, ensure_feature_name_type=True)
-        self._validate_feature_input(X, direction="transform")
-
-        lambdas_applied = list()
-
-        for i, _lambda in enumerate(self.lambda_list):
-
-            lambda_result = X.apply(func=_lambda, axis=0, raw=True)
-            lambda_result.columns = pd.Index(
-                [f"{feature_name}_lambda{i}" for feature_name in X.columns]
-            )
-
-            lambdas_applied.append(lambda_result)
-
-        X_transformed = pd.concat(lambdas_applied, axis=1)
-        X_transformed.columns.name = TSCDataFrame.tsc_feature_col_name
-
-        if isinstance(X, TSCDataFrame):
-            return TSCDataFrame(X_transformed)
-        else:
-            return X_transformed
-
-
 class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
-    """Compute principal components from data and also allow time series collection data.
+    """Compute principal components from data.
 
-    ``TSCPrincipalComponent`` subclasses ``PCA`` from scikit-learn to generalize the
-    input and output of pandas DataFrames (including TSCDataFrame). All input
+    This is a subclass of scikit-learn's ``PCA``  to generalize the
+    input and output of :class:`pandas.DataFrames` and :class:`.TSCDataFrame`. All input
     parameters remain the same. For documentation please visit:
 
-    * `PCA docu <https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html>`_ # noqa
+    * `PCA docu <https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html>`_
     * `PCA user guide <https://scikit-learn.org/stable/modules/decomposition.html#pca>`_
     """
 
@@ -374,12 +284,12 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
         )
 
     def fit(self, X: TransformType, y=None, **fit_params) -> "PCA":
-        """Compute the principal component vectors from training data.
+        """Compute the principal components from training data.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data with shape `(n_samples, n_features)`.
+            Training data of shape `(n_samples, n_features)`.
 
         y: None
             ignored
@@ -405,13 +315,13 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Out-of-sample points with shape `(n_samples, n_features)` to perform dimension
+            Out-of-sample points of shape `(n_samples, n_features)` to perform dimension
             reduction on.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_components_)`
+            same type as `X` of shape `(n_samples, n_components_)`
         """
 
         check_is_fitted(self)
@@ -419,15 +329,17 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
 
         self._validate_feature_input(X, direction="transform")
         pca_data = super(TSCPrincipalComponent, self).transform(self._X_to_numpy(X))
-        return self._same_type_X(X, values=pca_data, set_columns=self.features_out_[1])
+        return self._same_type_X(
+            X, values=pca_data, feature_names=self.features_out_[1]
+        )
 
-    def fit_transform(self, X: TransformType, y=None) -> TransformType:
+    def fit_transform(self, X: TransformType, y=None, **fit_params) -> TransformType:
         """Compute principal components from data and reduce dimension on same data.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data with shape `(n_samples, n_features)`.
+            Training data of shape `(n_samples, n_features)`.
             
         y: None
             ignored
@@ -435,7 +347,7 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_components_)`
+            same type as `X` of shape `(n_samples, n_components_)`
         """
 
         X = self._validate_data(X)
@@ -447,22 +359,22 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
             self._X_to_numpy(X), y=y
         )
         return self._same_type_X(
-            X, values=pca_values, set_columns=self.features_out_[1]
+            X, values=pca_values, feature_names=self.features_out_[1]
         )
 
     def inverse_transform(self, X: TransformType):
-        """Reconstruct the data to the original space.
+        """Map data from the reduced space back to the original space.
 
         Parameters
         ----------
         X:
-            Out-of-sample points with shape `(n_samples, n_components_)` to map back to
+            Out-of-sample points of shape `(n_samples, n_components_)` to map back to
             original space.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_features)`
+            same type as `X` of shape `(n_samples, n_features)`
         """
 
         self._validate_feature_input(X, direction="inverse_transform")
@@ -471,7 +383,7 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixIn):
         data_orig_space = super(TSCPrincipalComponent, self).inverse_transform(X_intern)
 
         return self._same_type_X(
-            X, values=data_orig_space, set_columns=self.features_in_[1]
+            X, values=data_orig_space, feature_names=self.features_in_[1]
         )
 
 
@@ -482,16 +394,18 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
     ----------
 
     lag
-        Number of time steps to lag before embedding past data.
+        Number of time steps to lag before embedding starts.
 
     delays
-        Number for delays to embed.
+        Number for time delays to embed.
 
     frequency
-        Time step frequency (every time step, every second, ...)
+        Time step frequency to emebd (e.g. to embed every sample or only every second
+        or third).
 
     Attributes
     ----------
+
     delay_indices_ : numpy.ndarray
         Delay indices (backwards in time) assuming a fixed time delta in the time series.
 
@@ -500,9 +414,17 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
         vector.
 
     delta_time_fit_
-        Time delta during measured during fit. This is primarily used to check that
+        Time delta measured during model fit. This is primarily used to check that
         `transform` or `inverse_transform` data still have the same time delta for
         consistency.
+
+    References
+    ----------
+
+    * Original paper from Takens :cite:`rand_detecting_1981`
+    * Takens delay embedding in the context of time series data
+      :cite:`champion_discovery_2019` (the time delay embedding is then a transform
+      function of :py:class:`.EDMD` dictionary).
     """
 
     def __init__(
@@ -578,13 +500,12 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
         )
 
     def fit(self, X: DataFrameType, y=None, **fit_params) -> "TSCTakensEmbedding":
-        """Compute delay indices based on settings and check if they work for the time
-        series provided.
+        """Compute delay indices based on settings and validate input with setting.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame
-            Time series collection to check `delay_indices_` with.
+            Time series collection to validate for time delay embedding.
 
         y: None
             ignored
@@ -597,7 +518,7 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
         Raises
         ------
         TSCException
-            Time series collection restrictions in **X**: (1) time delta must be constant
+            Time series collection requirements in `X`: (1) time delta must be constant
             (2) all time series must have the minimum number of time samples to obtain
             one sample in the time delay embedding.
 
@@ -635,18 +556,20 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame
-            Time series (collection).
+            Time series collection.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame
             Each time series is shortend by the number of samples required for the
-            delays. Type falls back to `pandas.DataFrame` if not a valid TSCDataFrame.
+            delays. The type can fall back to `pandas.DataFrame` if the result is not
+            not a valid :class:`.TSCDataFrame` anymore (this is a typical scenario for
+            time series initial conditions).
 
         Raises
         ------
         TSCException
-            Time series collection restrictions in `X`: (1) time delta must be constant
+            Time series collection requirements in `X`: (1) time delta must be constant
             (2) all time series must have the minimum number of time samples to obtain
             one sample in the time delay embedding.
         """
@@ -727,18 +650,18 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
         return X
 
     def inverse_transform(self, X: TransformType) -> TransformType:
-        """Remove time delayed features from columns of time delay embedded time
-        series collection.
+        """Remove time delayed feature columns of time delay embedded time series
+        collection.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame
-            time delayed data
+            Time delayed data of shape `(n_samples, n_features_embedded)`
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame
-            same type and shape as `X`
+            same type as `X` of shape `(n_samples, n_features_original)`
         """
         check_is_fitted(self)
 
@@ -755,20 +678,21 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixIn):
 
 
 class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
-    """Represent data in terms coefficients of a set of radial basis functions.
+    """Represent data in coefficients of radial basis functions.
 
     Parameters
     ----------
 
     kernel
-        Radial basis kernel to compute the coeffients with.
+        Radial basis kernel to compute the coefficients with.
 
     center_type
         Selection of what to take as centers during fit. 
 
-        * `all_data` - all data are centers
-        * `initial_condition` - take initial conditions as centers. Note in for this
-           case the data `X` during fit must be a ``TSCDataFrame``.
+        * `all_data` - all data points during fit are used as centers
+        * `initial_condition` - take the initial condition states as centers.
+           Note for this option the data `X` during fit must be of
+           type :class:`.TSCDataFrame`.
 
     exact_distance
         An inexact distance computation increases the performance at the cost of
@@ -781,9 +705,9 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
     centers_: numpy.ndarray
         The center points of the radial basis functions.
 
-    inv_coeff_matrix_: np.ndarray
-        Matrix to map RBF coefficients to original space. Computation is delayed
-        until `inverse_transform` is called for the first time.
+    inv_coeff_matrix_: numpy.ndarray
+        Matrix to map radial basis coefficients to original space. Computation is
+        delayed until `inverse_transform` is called for the first time.
     """
 
     _cls_valid_center_types = ["all_data", "initial_condition"]
@@ -811,8 +735,8 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Point centers with shape (n_centers, n_features). Must be `TSCDataFrame` if
-            center type is `initial_condition`.
+            Data of shape (n_centers, n_features) to extract point centers from. Must be
+            of type :class:`TSCDataFrame` if center type is `initial_condition`.
 
         y: None
             ignored
@@ -828,8 +752,6 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         X = self._validate_data(X)
         self._validate_center_type(center_type=self.center_type)
 
-        from datafold.pcfold import PCManifold
-
         if self.center_type == "all_data":
             self.centers_ = self._X_to_numpy(X)
         else:  # self.center_type == "initial_condition":
@@ -840,8 +762,7 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         self.centers_ = PCManifold(
             self.centers_,
             kernel=self.kernel,
-            dist_backend="brute",
-            **dict(exact_numeric=self.exact_distance),
+            dist_kwargs=dict(backend="brute", exact_numeric=self.exact_distance),
         )
 
         n_centers = self.centers_.shape[0]
@@ -850,17 +771,17 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         return self
 
     def transform(self, X: TransformType) -> TransformType:
-        """Transform data to coefficients of the radial basis functions.
+        """Transform data to radial basis functions coefficients.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Data with shape `(n_samples, n_features)`.
+            Data of shape `(n_samples, n_features)`.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_centers)`
+            same type as `X` of shape `(n_samples, n_centers)`
         """
         check_is_fitted(self, attributes=["centers_"])
         X = self._validate_data(X)
@@ -869,15 +790,17 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         X_intern = self._X_to_numpy(X)
         rbf_coeff = self.centers_.compute_kernel_matrix(Y=X_intern)
 
-        return self._same_type_X(X, values=rbf_coeff, set_columns=self.features_out_[1])
+        return self._same_type_X(
+            X, values=rbf_coeff, feature_names=self.features_out_[1]
+        )
 
     def fit_transform(self, X, y=None, **fit_params):
-        """Simultaneuously set the data to transform also to the point centers .
+        """Set the data as centers and transform to radial basis coefficients.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Radial basis center points and data to transform with shape \
+            Radial basis center points and data to transform of shape \
             `(n_samples, n_features)`
 
         y: None
@@ -886,7 +809,7 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_centers)`
+            same type as `X` of shape `(n_samples, n_centers)`
         """
         self.fit(X)
         X_intern = self._X_to_numpy(X)
@@ -899,7 +822,7 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
             rbf_coeff = self.centers_.compute_kernel_matrix(Y=X_intern)
 
         return self._same_type_X(
-            X=X, values=rbf_coeff, set_columns=self.features_out_[1]
+            X=X, values=rbf_coeff, feature_names=self.features_out_[1]
         )
 
     def inverse_transform(self, X: TransformType):
@@ -908,13 +831,13 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Coefficient representation of the radial basis functions with shape \
+            Coefficient representation of the radial basis functions of shape \
             `(n_samples, n_center)`.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_features)`
+            same type as `X` of shape `(n_samples, n_features)`
         """
         self._validate_feature_input(X, direction="inverse_transform")
 
@@ -931,41 +854,325 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixIn):
             )[0]
 
         X_inverse = rbf_coeff @ self.inv_coeff_matrix_
-        return self._same_type_X(X, values=X_inverse, set_columns=self.features_in_[1])
+        return self._same_type_X(
+            X, values=X_inverse, feature_names=self.features_in_[1]
+        )
+
+
+class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixIn):
+    """Compute polynomial features from data.
+
+    This is a subclass of ``PolynomialFeatures`` from scikit-learn to generalize the
+    input and output of :class:`pandas.DataFrames` and :class:`.TSCDataFrame`.
+
+    This class adds the parameter `include_first_order` to choose whether to include the
+    identity states. For all other parameters please visit the super class
+    documentation of
+    `PolynomialFeatures <https://scikit-learn.org/stable/modules/generated/sklearn.preprocessing.PolynomialFeatures.html>`_.
+    """
+
+    def __init__(
+        self,
+        degree: int = 2,
+        interaction_only: bool = False,
+        include_bias: bool = False,
+        include_first_order=False,
+    ):
+        self.include_first_order = include_first_order
+
+        super(TSCPolynomialFeatures, self).__init__(
+            degree=degree,
+            interaction_only=interaction_only,
+            include_bias=include_bias,
+            order="C",
+        )
+
+    @property
+    def powers_(self):
+        powers = super(TSCPolynomialFeatures, self).powers_
+        if self.include_first_order:
+            return powers
+        else:
+            return powers[powers.sum(axis=1) != 1, :]
+
+    def _get_poly_feature_names(self, X, input_features=None):
+        # Note: get_feature_names function is already provided by super class
+        if self._has_feature_names(X):
+            feature_names = self.get_feature_names(input_features=X.columns)
+        else:
+            feature_names = self.get_feature_names()
+        return feature_names
+
+    def _non_id_state_mask(self):
+        powers = super(TSCPolynomialFeatures, self).powers_
+        return powers.sum(axis=1) != 1
+
+    def fit(self, X: TransformType, y=None, **fit_params) -> "TSCPolynomialFeatures":
+        """Compute number of output features.
+
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Data of shape `(n_samples, n_features)`.
+
+        Returns
+        -------
+        TSCPolynomialFeatures
+            self
+        """
+        X = self._validate_data(X)
+
+        super(TSCPolynomialFeatures, self).fit(X, y=y)
+
+        self._setup_features_fit(
+            X, features_out=self._get_poly_feature_names(X),
+        )
+
+        return self
+
+    def transform(self, X: TransformType) -> TransformType:
+        """Transform data to polynomial features.
+
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            The data of shape `(n_samples, n_features)` to transform.
+
+        Returns
+        -------
+        TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Transformed data of shape `(n_samples, n_polynomials)` and with same type
+            as `X`.
+        """
+        check_is_fitted(self)
+        X = self._validate_data(X)
+        self._validate_feature_input(X, direction="transform")
+
+        poly_data = super(TSCPolynomialFeatures, self).transform(X)
+
+        if not self.include_first_order:
+            poly_data = poly_data[:, self._non_id_state_mask()]
+
+        poly_data = self._same_type_X(
+            X, values=poly_data, feature_names=self._get_poly_feature_names(X)
+        )
+
+        return poly_data
+
+
+class TSCApplyLambdas(BaseEstimator, TSCTransformerMixIn):
+    """Transform data in an element-by-element fashion with lambda functions.
+
+    Each function is called on every column in the data (i.e. the number of samples
+    remains the same).
+
+    Two examples using a Python lambda expression and a NumPy's
+    `ufunc <https://numpy.org/devdocs/reference/ufuncs.html>`_:
+
+    .. code-block:: python
+
+        TSCApplyLambdas(lambdas=[lambda x: x**3])
+        TSCApplyLambdas(lambdas=[np.sin, np.cos])
+
+    Parameters
+    ----------
+    lambdas
+        List of `lambda` or `ufunc` functions (`ufunc` should not be reducing the data).
+        Each column `X_col` is passed to the function and the returned `X_transformed`
+        data must be of the same shape as `X_col`, i.e.
+        :code:`X_transformed = func(X_col)`
+    """
+
+    def __init__(self, lambdas):
+        self.lambdas = lambdas
+
+    def _not_implemented_numpy_arrays(self, X):
+        if isinstance(X, np.ndarray):
+            raise NotImplementedError(
+                "Currently not implemented for numpy.ndarray. If this is required please "
+                "open an issue on Gitlab."
+            )
+
+    def fit(self, X: TransformType, y=None, **fit_params) -> "TSCApplyLambdas":
+        """Set internal feature information.
+
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Training data.
+        y: None
+            ignored
+
+        Returns
+        -------
+        TSCApplyLambdas
+            self
+        """
+        self._not_implemented_numpy_arrays(X)
+        X = self._validate_data(X, ensure_feature_name_type=True)
+
+        features_out = [
+            f"{feature_name}_lambda{i}"
+            for feature_name in X.columns
+            for i in range(len(self.lambdas))
+        ]
+
+        self._setup_features_fit(X, features_out=features_out)
+        return self
+
+    def transform(self, X: TransformType) -> TransformType:
+        """Transform data with specified lambda functions.
+
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Data of shape `(n_samples, n_features)` to transform.
+
+        Returns
+        -------
+        TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            The transformed data of same type as `X` and of shape
+            `(n_samples, n_lambdas * n_features)`
+        """
+        self._not_implemented_numpy_arrays(X)
+
+        check_is_fitted(self)
+        X = self._validate_data(X, ensure_feature_name_type=True)
+        self._validate_feature_input(X, direction="transform")
+
+        lambdas_applied = list()
+
+        for i, _lambda in enumerate(self.lambdas):
+
+            lambda_result = X.apply(func=_lambda, axis=0, raw=True)
+            lambda_result.columns = pd.Index(
+                [f"{feature_name}_lambda{i}" for feature_name in X.columns]
+            )
+
+            lambdas_applied.append(lambda_result)
+
+        X_transformed = pd.concat(lambdas_applied, axis=1)
+        X_transformed.columns.name = TSCDataFrame.tsc_feature_col_name
+
+        if isinstance(X, TSCDataFrame):
+            return TSCDataFrame(X_transformed)
+        else:
+            return X_transformed
 
 
-@warn_experimental_class
 class TSCFiniteDifference(BaseEstimator, TSCTransformerMixIn):
-    def __init__(self, spacing: Union[str, float] = "dt", diff_order=1, accuracy=2):
+    """Compute time derivative with finite difference scheme.
+
+    .. note::
+        The class internally uses the Python package findiff, which currently is
+        optional in *datafold*. The class raises an `ImportError` if findiff is not
+        installed.
+
+    Parameters
+    ----------
+
+    spacing: Union[str, float]
+        The time difference between samples. If "dt" (str) then the time sampling
+        frequency of a :meth:`.TSCDataFrame.delta_time` is used during fit.
+
+    diff_order
+        The derivative order.
+
+    accuracy
+        The convergence order of the finite difference scheme.
+
+    Attributes
+    ----------
+
+    spacing_
+        The resolved time difference between samples. Equals the parameter
+        input if it was of type :class`float`.
+
+    See Also
+    --------
+    `findiff documentation <https://findiff.readthedocs.io/en/latest/>`_
+    """
+
+    def __init__(
+        self, spacing: Union[str, float] = "dt", diff_order: int = 1, accuracy: int = 2
+    ):
+
+        if not IMPORTED_FINDIFF:
+            # TODO: Currently, findiff is an optional package and listed in the
+            #  dev-requirements. If the class is more used internally of datafold,
+            #  make it a proper dependency.
+            raise ImportError(
+                "TSCFiniteDifference requires the Python package "
+                "'findiff' installed."
+            )
+
         self.spacing = spacing
         self.diff_order = diff_order
         self.accuracy = accuracy
 
-    def fit(self, X: TransformType, y=None, **fit_params):
+    def fit(self, X: TransformType, y=None, **fit_params) -> "TSCFiniteDifference":
+        """Set and validate time spacing between samples.
 
-        X = self._validate_data(X, ensure_feature_name_type=False)
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Data of shape `(n_samples, n_features)`.
+
+        y: None
+            ignored
+
+        Returns
+        -------
+        TSCFiniteDifference
+            self
+
+        Raises
+        ------
+        TSCException
+            If time series data has not a constant time delta or the input `X` has not
+            the same value as specified in `spacing` during initialization.
+
+        """
+        X = self._validate_data(
+            X,
+            ensure_feature_name_type=False,
+            validate_tsc_kwargs=dict(
+                ensure_delta_time=self.spacing
+                if isinstance(self.spacing, float)
+                else None
+            ),
+        )
 
         if self._has_feature_names(X):
-            features_out = [f"{col}_dt" for col in X.columns]
+            features_out = [f"{col}_dot" for col in X.columns]
         else:
-            features_out = [f"dt{i}" for i in np.arange(X.shape[1])]
+            features_out = [f"dot{i}" for i in np.arange(X.shape[1])]
 
         self._setup_features_fit(X=X, features_out=features_out)
 
         if self.spacing == "dt":
 
             if not isinstance(X, TSCDataFrame):
-                raise TypeError("'spacing=dt' only works for time series collections")
+                raise TypeError(
+                    "For input 'spacing=dt' a time series collections is required."
+                )
 
             self.spacing_ = X.delta_time
 
             if isinstance(self.spacing_, pd.Series) or np.isnan(self.spacing_):
-                raise ValueError(
-                    "delta time (=spacing) of TSCDataFrame must be constant"
-                )
+                raise TSCException.not_const_delta_time(actual_delta_time=self.spacing_)
         else:
             self.spacing_ = self.spacing
 
+            if (
+                isinstance(X, TSCDataFrame)
+                and np.asarray(self.spacing_ != X.delta_time).all()
+            ):
+                raise ValueError(
+                    f"A spacing of {self.spacing} was specified, but the time series "
+                    f"collection has a time delta of {X.delta_time}"
+                )
+
         check_scalar(
             self.spacing_,
             "spacing",
@@ -994,11 +1201,27 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixIn):
         return self
 
     def transform(self, X: TransformType) -> TransformType:
+        """Compute the finite difference values.
 
-        from findiff import FinDiff
+        Parameters
+        ----------
+        X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Data of shape `(n_samples, n_features)`.
 
+        Returns
+        -------
+        TSCDataFrame, pandas.DataFrame, numpy.ndarray
+            Transformed data of same shape and type as `X`.
+
+        Raises
+        ------
+        TSCException
+            If input `X` has a different time delta than data during `fit`.
+        """
         check_is_fitted(self)
-        X = self._validate_data(X)
+        X = self._validate_data(
+            X, validate_tsc_kwargs=dict(ensure_delta_time=self.spacing_)
+        )
         self._validate_feature_input(X=X, direction="transform")
 
         # first parameter is the axis along which to take the derivative
@@ -1016,5 +1239,5 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixIn):
             time_derivative = dt_func(np.asarray(X))
 
         return self._same_type_X(
-            X=X, values=time_derivative, set_columns=self.features_out_[1]
+            X=X, values=time_derivative, feature_names=self.features_out_[1]
         )
