@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 import abc
-import logging
+import warnings
 from typing import Optional, Sequence, Type, Union
 
 import numexpr as ne
@@ -10,10 +10,10 @@ import scipy.sparse
 import scipy.spatial
 from scipy.spatial.distance import cdist, pdist, squareform
 from sklearn.metrics import pairwise_distances
-from sklearn.neighbors import NearestNeighbors
+from sklearn.neighbors import BallTree, NearestNeighbors
 
 from datafold.decorators import warn_experimental_function
-from datafold.utils.general import if1dim_colvec
+from datafold.utils.general import if1dim_colvec, if1dim_rowvec, is_symmetric_matrix
 
 try:
     # rdist is an optional distance algorithm backend -- an import error is raised only
@@ -25,11 +25,20 @@ except ImportError:
     rdist = None
     IS_IMPORTED_RDIST = False
 
-logger = logging.getLogger(__name__)
-
 
 class DistanceAlgorithm(metaclass=abc.ABCMeta):
-    """Abstract class to warp or implement distance matrix algorithms (dense or sparse).
+    """Abstract base class for distance matrix algorithms (dense or sparse).
+
+    Important aspects and conventions for the distance algorithms:
+
+    * The terms "pair-wise" (pdist) and "component-wise" (cdist) are
+      adapted from the scipy's distance matrix computations
+      :class:`scipy.sparse.spatial.pdist` and :class:`scipy.sparse.spatial.cdist`
+    * A sparse distance matrix with a distance cut-off value does not store distance
+      values *above* the cut-off. Importantly, this means that the sparse matrix
+      **must** store real distance zeros (duplicates or self-distances in case of
+      `pdist`) and treat the zeros not stored in the sparse matrix as "distance
+      values out of range".
 
     Parameters
     ----------
@@ -113,12 +122,13 @@ class DistanceAlgorithm(metaclass=abc.ABCMeta):
     def pdist(
         self, X: np.ndarray, cut_off: Optional[float] = None, **backend_options
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
-        """Abstract method: Computes the distance matrix pair-wise from the dataset.
+        """Abstract method to compute the distance matrix pair-wise from the dataset.
 
-        From a pairwise computation follow the matrix properties:
+        In a pair-wise computation the query and reference data are the same. From this
+        the following distance matrix properties follow:
 
         * square
-        * diagonal contains distance to itself and is therefore zero
+        * diagonal contains distance to itself and are therefore zero
         * symmetric
         """
 
@@ -130,106 +140,32 @@ class DistanceAlgorithm(metaclass=abc.ABCMeta):
         cut_off: Optional[float] = None,
         **backend_options,
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
-        """Abstract method: Computes the distance matrix component-wise between two
-        data sets.
+        """Abstract method to compute the distance matrix component-wise.
 
-        From a component-wise distance computation follow the general matrix properties:
+        The dataset `X` refers to the reference dataset, and the distances are
+        computed component-wise for the query dataset `Y`. From this the general
+        matrix properties follow:
 
-        * rectangular matrix with shape (n_samples_Y, n_samples_X)
+        * rectangular matrix of shape `(n_samples_Y, n_samples_X)`
         * outlier points can lead to zero columns / rows
+        * only duplicated between `X` and `Y` have zero entries
         """
 
 
-class BruteForceNumexpr(DistanceAlgorithm):
-    """
-    Source of algorithm:
-    https://stackoverflow.com/questions/47271662/what-is-the-fastest-way-to-compute-an-rbf-kernel-in-python
-    """
-
-    backend_name = None  #  TODO: algorithm not used at the moment
-
-    def __init__(self, metric):
-        if metric not in ["euclidean", "sqeuclidean"]:
-            raise ValueError
-
-        super(BruteForceNumexpr, self).__init__(metric=metric)
-
-    def _eval_numexpr_dist(self, A, B, C):
-
-        distance_matrix = ne.evaluate(
-            f"A + B - 2. * C",
-            {"A": A, "B": B, "C": C,},
-            optimization="aggressive",
-            order="C",
-        )
-
-        # For some reason actual zero values can be slightly negative (in range of
-        # numerical noise ~1e-14) --> this then results into nan values when applying
-        # the sqrt() function for the euclidean
-        distance_matrix[distance_matrix < 0] = 0
-        return distance_matrix
-
-    def pdist(
-        self, X: np.ndarray, cut_off: Optional[float] = None, **backend_options
-    ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
-
-        X_norm = ne.evaluate("sum(X ** 2, axis=1)", {"X": X})
-        # X_norm = np.linalg.norm(X, axis=1)
-        # X_norm = np.square(X_norm, X_norm)
-
-        distance_matrix = self._eval_numexpr_dist(
-            A=X_norm[np.newaxis, :], B=X_norm[:, np.newaxis], C=np.dot(X, X.T),
-        )
-
-        if self.metric == "euclidean":
-            distance_matrix = np.sqrt(distance_matrix, out=distance_matrix)
-
-        # Somehow, zero valuess are often imprecise,
-        # For these zero values are easy to set:
-        np.fill_diagonal(distance_matrix, 0)
-
-        # Brute force algorithms can only sparsify the distance matrix, after everything
-        # is computed:
-        if cut_off is not None:
-            distance_matrix = self._dense2csr_matrix(distance_matrix, cut_off)
-
-        return distance_matrix
-
-    def cdist(
-        self,
-        X: np.ndarray,
-        Y: np.ndarray,
-        cut_off: Optional[float] = None,
-        **backend_options,
-    ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
-
-        X_norm = ne.evaluate("sum(X ** 2, axis=1)", {"X": X})
-        Y_norm = ne.evaluate("sum(Y ** 2, axis=1)", {"Y": Y})
-
-        distance_matrix = self._eval_numexpr_dist(
-            A=Y_norm[:, np.newaxis], B=X_norm[np.newaxis, :], C=np.dot(Y, X.T)
-        )
-
-        if self.metric == "euclidean":
-            distance_matrix = np.sqrt(distance_matrix, out=distance_matrix)
-
-        if cut_off is not None:
-            distance_matrix = self._dense2csr_matrix(distance_matrix, cut_off)
-
-        return distance_matrix
-
-
 class BruteForceDist(DistanceAlgorithm):
-    """Computes the full distance matrix.
+    """Computes all distance pairs in the distance matrix.
 
     Chooses either
 
-        * `scipy.pdist <https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.pdist.html>_` \
+        * `scipy.pdist <https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.pdist.html>`_ \
            and
            `scipy.cdist <https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.cdist.html>`_
         * `sklearn.pairwise_distances <https://scikit-learn.org/stable/modules/generated/sklearn.metrics.pairwise_distances.html>`_
 
-        Depending on the parameter `exact_numeric`.
+        Depending on the parameter `metric` and argument `exact_numeric`.
+
+        For an explanation of how `exact_numeric = False` is beneficial, see the
+        `scikit-learn` `documentation <https://scikit-learn.org/stable/modules/generated/sklearn.metrics.pairwise.euclidean_distances.html>`_
 
     Parameters
     ----------
@@ -255,20 +191,20 @@ class BruteForceDist(DistanceAlgorithm):
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples, n_features)`
+            Point cloud of shape `(n_samples, n_features)`.
 
         cut_off
             distances larger than `cut_off` are set to zero
 
             .. note::
-                Distances with larger distance are removed after they memory was
-                allocated and distances computed. It is recommended to use other
-                distance algorithms that directly integrates cut off sparsity .
+                Distances with larger distance are removed after a full memory
+                allocation of the distance matrix. It is recommended to use
+                distance algorithms that directly integrate a cut-off sparsity.
 
         exact_numeric
-            If False, computes (also depending from metric) distances more efficiently
-            at the cost of introducing numerical noise. (empirically: `~1e-14` for
-            "sqeuclidean" and `~1e-7` for "euclidean" metric).
+            If False, computes Euclidean distances more efficiently
+            at the cost of introducing numerical noise. Empirically `~1e-14` for
+            "sqeuclidean" and `~1e-7` for "euclidean" metric.
             
         **backend_options
             Keyword arguments handled to the executing backend.
@@ -276,7 +212,7 @@ class BruteForceDist(DistanceAlgorithm):
         Returns
         -------
         Union[numpy.ndarray, scipy.sparse.csr_matrix]
-            distance matrix with shape `(n_samples, n_samples)`
+            distance matrix of shape `(n_samples, n_samples)`
         """
         if exact_numeric:
             X = np.array(X)
@@ -301,20 +237,20 @@ class BruteForceDist(DistanceAlgorithm):
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
         """Component-wise distance matrix computation.
 
-        For not documented parameters look in ``pdist``
+        For undocumented parameters look at :meth:`.pdist`.
 
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples_X, n_features_X)`
+            Point cloud of shape `(n_samples_X, n_features_X)`.
 
         Y
-            point cloud with shape `(n_samples_Y, n_features_Y)`
+            Point cloud of shape `(n_samples_Y, n_features_Y)`.
 
         Returns
         -------
         Union[numpy.ndarray, scipy.sparse.csr_matrix]
-            distance matrix with shape `(n_samples_Y, n_samples_X)`
+            distance matrix of shape `(n_samples_Y, n_samples_X)`
         """
 
         if exact_numeric:
@@ -332,10 +268,11 @@ class BruteForceDist(DistanceAlgorithm):
 
 
 class RDist(DistanceAlgorithm):
-    """Sparse distance matrix algorithm rdist, targeting point clouds with
-    manifold assumption.
+    """Sparse distance matrix algorithm rdist, for point clouds with manifold assumption.
 
-    The dependency on Python package "rdist" is optional.
+    .. note::
+        The dependency on the Python package is optional. The package is currentl not
+        published.
 
     Parameters
     ----------
@@ -380,7 +317,7 @@ class RDist(DistanceAlgorithm):
         return max_distance
 
     def _get_dist_options(self):
-        return {"strict_distance": True, "max_incr_radius": 0, "kmin": 0}
+        return {"max_incr_radius": 0, "kmin": 0}
 
     def pdist(
         self, X: np.ndarray, cut_off: Optional[float] = None, **backend_options
@@ -390,10 +327,11 @@ class RDist(DistanceAlgorithm):
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples, n_features)`
+            Point cloud of shape `(n_samples, n_features)`.
 
         cut_off
-            distances (always Euclidean) larger than `cut_off` are set to zero
+            Distance values (always Euclidean metric) that are larger are not stored in
+            distance matrix.
 
         **backend_options
             keywords handled to build
@@ -401,7 +339,7 @@ class RDist(DistanceAlgorithm):
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples, n_samples)`
+            distance matrix of shape `(n_samples, n_samples)`
         """
 
         max_distance = self._numeric_cut_off(cut_off)
@@ -430,20 +368,20 @@ class RDist(DistanceAlgorithm):
     ) -> scipy.sparse.csr_matrix:
         """Component-wise distance matrix computation.
 
-        For not documented parameters look in ``pdist``.
+        For undocumented parameters look at :meth:`.pdist`.
 
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples_X, n_features_X)`
+            Point cloud of shape `(n_samples_X, n_features_X)`.
 
         Y
-            point cloud with shape `(n_samples_Y, n_features_Y)`
+            Point cloud of shape `(n_samples_Y, n_features_Y)`.
 
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples_Y, n_samples_X)`
+            distance matrix of shape `(n_samples_Y, n_samples_X)`
         """
 
         max_distance = self._numeric_cut_off(cut_off)
@@ -461,7 +399,7 @@ class RDist(DistanceAlgorithm):
 
 
 class ScipyKdTreeDist(DistanceAlgorithm):
-    """Sparse distance matrix computation using scipy's KD-tree implementation.
+    """Sparse distance matrix computation using scipy's kd-tree implementation.
 
     Parameters
     ----------
@@ -472,8 +410,8 @@ class ScipyKdTreeDist(DistanceAlgorithm):
     ----------
 
     :class:`scipy.spatial.cKDTree`
-    
     :meth:`scipy.spatial.KDTree.sparse_distance_matrix`
+
     """
 
     backend_name = "scipy.kdtree"
@@ -502,10 +440,11 @@ class ScipyKdTreeDist(DistanceAlgorithm):
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples, n_features)`
+            Point cloud of shape `(n_samples, n_features)`.
 
         cut_off
-            larger distances (in Euclidean metric) are set to zero
+            Distance values (always Euclidean metric) that are larger are not stored in
+            distance matrix.
 
         **backend_options
             key word arguments handled to `cKDTree`
@@ -513,7 +452,7 @@ class ScipyKdTreeDist(DistanceAlgorithm):
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples, n_samples)`
+            distance matrix of shape `(n_samples, n_samples)`
         """
 
         # TODO: if necessary there are build_options and compute_options required,
@@ -540,20 +479,18 @@ class ScipyKdTreeDist(DistanceAlgorithm):
     ) -> scipy.sparse.csr_matrix:
         """Component-wise distance matrix computation.
 
-        For not documented parameters look in ``pdist``.
-
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples_X, n_features_X)`
+            Point cloud of shape `(n_samples_X, n_features_X)`.
 
         Y
-            point cloud with shape `(n_samples_Y, n_features_Y)`
+            Point cloud of shape `(n_samples_Y, n_features_Y)`.
 
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples_Y, n_samples_X)`
+            distance matrix of shape `(n_samples_Y, n_samples_X)`
         """
         max_distance = self._numeric_cut_off(cut_off)
         max_distance = self._adapt_correct_metric_max_distance(max_distance)
@@ -572,7 +509,7 @@ class ScipyKdTreeDist(DistanceAlgorithm):
 
 
 class SklearnBalltreeDist(DistanceAlgorithm):
-    """Ball tree implementation from scikit-learn.
+    """Distance matrix using ball tree implementation from scikit-learn.
 
     Parameters
     ----------
@@ -606,11 +543,12 @@ class SklearnBalltreeDist(DistanceAlgorithm):
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples, n_features)`
+            Point cloud of shape `(n_samples, n_features)`.
 
         cut_off
-            larger distances are set to zero (see
-            :class:`sklearn.neighbors.NearestNeighbors` documentation)
+            Distance values (always Euclidean metric) that are larger are not stored in
+            distance matrix. (see also :class:`sklearn.neighbors.NearestNeighbors`
+            documentation)
 
         **backend_options
             handled to `NearestNeighbor`
@@ -618,7 +556,7 @@ class SklearnBalltreeDist(DistanceAlgorithm):
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples, n_samples)`
+            distance matrix of shape `(n_samples, n_samples)`
         """
         metric, cut_off = self._map_metric_and_cut_off(cut_off)
 
@@ -647,20 +585,20 @@ class SklearnBalltreeDist(DistanceAlgorithm):
     ) -> scipy.sparse.csr_matrix:
         """Component-wise distance matrix computation.
 
-        For not documented parameters look in ``pdist``.
+        For undocumented parameters look at :meth:`.pdist`.
 
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples_X, n_features_X)`
+            Point cloud of shape `(n_samples_X, n_features_X)`.
 
         Y
-            point cloud with shape `(n_samples_Y, n_features_Y)`
+            Point cloud of shape `(n_samples_Y, n_features_Y)`.
 
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples_Y, n_samples_X)`
+            distance matrix of shape `(n_samples_Y, n_samples_X)`
         """
         metric, cut_off = self._map_metric_and_cut_off(cut_off)
 
@@ -721,18 +659,19 @@ class GuessOptimalDist(DistanceAlgorithm):
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples, n_features)`
+            Point cloud of shape `(n_samples, n_features)`.
 
         cut_off
-            larger distances are set to zero
+            Distance values (always Euclidean metric) that are larger are not stored in
+            distance matrix.
 
         **backend_options
-            keyword arguments handled to guessed optimal :meth:`DistanceAlgorithm.pdist`
+            Keyword arguments passed to :meth:`DistanceAlgorithm.pdist`
         
         Returns
         -------
         scipy.sparse.csr_matrix
-            distance matrix with shape `(n_samples, n_samples)`
+            distance matrix of shape `(n_samples, n_samples)`
         """
         return self._guess_optimal_backend(cut_off).pdist(X, cut_off, **backend_options)
 
@@ -745,20 +684,20 @@ class GuessOptimalDist(DistanceAlgorithm):
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
         """Component-wise distance matrix computation.
 
-        For not documented parameters look in ``pdist``.
+        For undocumented parameters look at :meth:`.pdist`.
 
         Parameters
         ----------
         X
-            point cloud with shape `(n_samples_X, n_features_X)`
+            Point cloud of shape `(n_samples_X, n_features_X)`.
 
         Y
-            point cloud with shape `(n_samples_Y, n_features_Y)`
+            Point cloud of shape `(n_samples_Y, n_features_Y)`.
 
         Returns
         -------
         Union[numpy.ndarray, scipy.sparse.csr_matrix]
-            distance matrix with shape `(n_samples_Y, n_samples_X)`
+            distance matrix of shape `(n_samples_Y, n_samples_X)`
         """
         return self._guess_optimal_backend(cut_off).cdist(
             X, Y, cut_off, **backend_options
@@ -769,7 +708,8 @@ def _k_smallest_element_value(
     distance_matrix, k: int, ignore_zeros: bool = True, fill_value: float = 0.0
 ):
     """Compute the k-th smallest element of distance matrix, i.e. the element where only
-    k-1 elements are smaller. If ignore_zeros=True only positive distances are considered.
+    k-1 elements are smaller. If `ignore_zeros=True` only positive distances are
+    considered.
     """
 
     if k > distance_matrix.shape[1] or k < 0:
@@ -818,6 +758,123 @@ def _k_smallest_element_value(
     return k_smallest_values
 
 
+def _ensure_kmin_nearest_neighbor(
+    X: np.ndarray,
+    Y: Optional[np.ndarray],
+    metric: str,
+    kmin: int,
+    distance_matrix: scipy.sparse.csr_matrix,
+) -> scipy.sparse.csr_matrix:
+    """Computes `kmin` nearest neighbors for all points that in the current distance
+    matrix have not at least `kmin` neighbors, yet.
+
+    This function is especially useful to make sure that a neighborhood graph (
+    described by a distance matrix) is fully connected. If outlier have no
+    (or only self neighbor), then this can have unwanted side effects in some
+    applications.
+
+    Internally, the k-NN query is carried out using :class:`sklearn.neighbors.BallTree`.
+
+    Parameters
+    ----------
+    X
+        Point cloud of shape `(n_samples_X, n_features_X)`.
+
+    Y
+        Query Point cloud of shape `(n_samples_Y, n_features_Y)`. If not given,
+        then `Y=X` (pdist case).
+
+    metric
+        distance metric
+
+    kmin
+        Minimum number of neighbors. Note, for the `pdist` case, `kmin==1` is already
+        fulfilled by the diagonal line (self-distances).
+
+    distance_matrix
+        Current distance matrix to which the missin distance pairs are inserted.
+
+    Returns
+    -------
+    scipy.sparse.csr_matrix
+        distance matrix with shape `(n_samples_Y, n_samples_X)`
+    """
+
+    current_nnz = distance_matrix.getnnz(axis=1)
+    knn_query_indices = np.where(current_nnz < kmin)[0]
+    is_pdist = Y is None
+
+    if len(knn_query_indices) != 0:
+
+        if is_pdist:
+            Y = X.view()
+        else:
+            assert isinstance(Y, np.ndarray)
+            if (
+                Y.shape[0] != distance_matrix.shape[0]
+                or X.shape[0] != distance_matrix.shape[1]
+            ):
+                raise ValueError("Mismatch between dataset and distance matrix.")
+
+        _ball_tree = BallTree(X, leaf_size=40, metric=metric)
+        distances, columns_indices = _ball_tree.query(
+            Y[knn_query_indices, :],
+            k=kmin,
+            return_distance=True,
+            dualtree=False,
+            breadth_first=False,
+            sort_results=False,
+        )
+
+        distances = np.reshape(
+            distances, newshape=np.product(distances.shape), order="C"
+        )
+
+        # Note: duplicates and trivial self-distances in the pdist are assumed to already
+        # covered by the DistanceAlgorithm (always contained in the radius!)
+        nnz_distance_mask = (distances != 0).astype(np.bool)
+        distances = distances[nnz_distance_mask]
+
+        knn_query_indices = np.repeat(knn_query_indices, kmin)[nnz_distance_mask]
+
+        columns_indices = np.reshape(
+            columns_indices, newshape=np.product(columns_indices.shape), order="C"
+        )[nnz_distance_mask]
+
+        if is_pdist:
+
+            knn_query_indices, columns_indices, distances = np.unique(
+                np.vstack(
+                    [
+                        np.column_stack(
+                            [knn_query_indices, columns_indices, distances]
+                        ),
+                        np.column_stack(
+                            [columns_indices, knn_query_indices, distances]
+                        ),
+                    ]
+                ),
+                axis=0,
+            ).T
+
+        kmin_elements_csr = scipy.sparse.csr_matrix(
+            (distances, (knn_query_indices, columns_indices)),
+            shape=distance_matrix.shape,
+        )
+
+        # TODO: This changes the sparsity structure and raises a warning. I am not sure
+        #  how to do make it right. For this attempt the tests fail:
+        #  distance_matrix.tolil(copy=False)[
+        #      kmin_elements_csr.nonzero()
+        #  ] = kmin_elements_csr.data
+        #  maybe the best is to combine the elements of kmin_elements_csr and distance
+        #  matrix into one set (sorting out the upper triangle for pdist) and then
+        #  create a new sparse matrix...
+        distance_matrix[kmin_elements_csr.nonzero()] = kmin_elements_csr.data
+
+    return distance_matrix.tocsr()
+
+
 def _all_available_distance_algorithm():
     """Searches for valid subclasses of :py:class:`DistanceAlgorithm`
 
@@ -851,7 +908,7 @@ def _all_available_distance_algorithm():
     return return_backends
 
 
-@warn_experimental_function
+@DeprecationWarning
 def apply_continuous_nearest_neighbor(distance_matrix, kmin, tol):
 
     if tol == 0:
@@ -888,7 +945,7 @@ def apply_continuous_nearest_neighbor(distance_matrix, kmin, tol):
 
 
 def get_backend_distance_algorithm(backend):
-    """Selects and validates the backend class to compute a distance matrix.
+    """Selects and validates the backend class for distance matrix computation.
 
     Parameters
     ----------
@@ -925,24 +982,23 @@ def compute_distance_matrix(
     metric: str = "euclidean",
     cut_off: Optional[float] = None,
     kmin: int = 0,
-    tol: float = 1.0,
     backend: Union[str, Type[DistanceAlgorithm]] = "brute",
     **backend_kwargs,
 ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
-    """Function to compute distance matrix with different settings and backends.
+    """Compute distance matrix with different settings and backends.
 
     Parameters
     ----------
 
     X
-        point cloud with shape `(n_samples_X, n_features_X)`
+        Point cloud of shape `(n_samples_X, n_features_X)`.
 
     Y
-        reference point cloud for component-wise computation with shape \
-        `(n_samples_Y, n_features_Y)`
+        Reference point cloud for component-wise computation of shape \
+        `(n_samples_Y, n_features_Y)`. If not given, then `Y=X` (pairwise computation)
     
     metric
-        distance metric (needs to be supported by backend)
+        Distance metric. Needs to be supported by backend.
 
     cut_off
         Distances larger than `cut_off` are set to zero and controls the degree of
@@ -950,65 +1006,72 @@ def compute_distance_matrix(
 
         .. note::
             The pseudo-metric "sqeuclidean" is handled differently in a way that the
-            cut off must be stated in in Eucledian distance (not squared cut off).
+            cut-off must be stated in in Eucledian distance (not squared cut-off).
 
     kmin
-        input for continuous nearest neighbor
-
-    tol
-        tolerance used in continuous nearest neighbors
+        Minimum number of neighbors. Ignored if `cut_off=np.inf` to indicate a dense
+        distance matrix, where all distance pairs are computed.
 
     backend
-        backend to compute distance matrix
+        Backend to compute distance matrix.
 
     **backend_kwargs
-        keyword agruments handled to selected backend
+        Keyword arguments handled to selected backend.
 
     Returns
     -------
     Union[numpy.ndarray, scipy.sparse.csr_matrix]
-        distance matrix with shape `(n_samples_X, n_samples_X)` if `Y=None`, \
-        else with shape `(n_samples_Y, n_samples_X)`
+        distance matrix of shape `(n_samples_X, n_samples_X)` if `Y=None`, \
+        else of shape `(n_samples_Y, n_samples_X)`
     """
 
     if not isinstance(X, np.ndarray):
         X = np.asarray(X)
-
     X = if1dim_colvec(X)
 
+    if Y is not None and not isinstance(Y, np.ndarray):
+        Y = np.asarray(Y)
+
+    if Y is not None:
+        Y = if1dim_rowvec(Y)
+
+        if X.shape[1] != Y.shape[1]:
+            raise ValueError(
+                "mismatch of point dimension: "
+                f"X.shape[1]={X.shape[1]} != Y.shape[1]={Y.shape[1]} "
+            )
+
     if X.shape[0] <= 1:
         raise ValueError(
             f"number of samples has to be greater than 1. Got {X.shape[0]}"
         )
 
-    logger.info("Setting up computation of distance matrix.")
+    if cut_off is not None:
+        if cut_off <= 0:
+            raise ValueError(f"cut_off={cut_off} must be a positive float.")
 
-    if cut_off is not None and np.isinf(cut_off):
-        # use dense case if cut_off is infinite
-        cut_off = None
+        try:
+            cut_off = float(cut_off)  # make sure to only deal with Python built-in
+        except:
+            raise TypeError(f"type(cut_off)={type(cut_off)} must be of type float")
+
+        if np.isinf(cut_off):
+            # use dense case if cut_off is infinite
+            cut_off = None
 
     is_pdist = Y is None
     is_sparse = cut_off is not None
 
-    if not is_sparse and kmin > 0:
-        # TODO: (raise error early, before expensive distance matrix is computed)
-        raise NotImplementedError(
-            "fix_kmin_rows currently only works for sparse distance matrices (i.e. with "
-            "set cut_off)."
-        )
-
     if metric == "sqeuclidean":
         if cut_off is not None:
             # NOTE: this is a special case. Usually the cut_off is represented in the
             # respective metric. However, for the 'sqeuclidean' case we use the
-            # 'euclidean' metric for the cut off.
+            # 'euclidean' metric for the cut-off.
             cut_off = cut_off ** 2
 
     backend_class = get_backend_distance_algorithm(backend)
     distance_method = backend_class(metric=metric)
 
-    logger.info(f"Start computing distance matrix.")
-
     if is_pdist:
         distance_matrix = distance_method.pdist(X, cut_off, **backend_kwargs)
     else:  # cdist
@@ -1018,32 +1081,43 @@ def compute_distance_matrix(
         # dense case stored in a sparse distance matrix -> convert to np.ndarray
         distance_matrix = distance_matrix.toarray()
 
-    if is_sparse and not scipy.sparse.issparse(distance_matrix):
-        raise RuntimeError(
-            "Distance_matrix is expected to be sparse but returned "
-            "distance matrix is dense. Please report bug. "
-        )
+    if is_sparse:
 
-    if is_sparse and distance_matrix.nnz == np.product(distance_matrix.shape):
-        logger.info(
-            f"cut_off={cut_off} value has no effect on sparsity of distance matrix, "
-            f"the sparse matrix is effectively dense."
-        )
+        if not scipy.sparse.issparse(distance_matrix):
+            raise RuntimeError(
+                "Distance_matrix is expected to be sparse but DistanceAlgorithm "
+                f"{backend} returned dense matrix. Please report bug."
+            )
 
-    if kmin > 0:
-        logger.info("apply continuous nearest neighbor on distance matrix")
-        distance_matrix = apply_continuous_nearest_neighbor(distance_matrix, kmin, tol)
-
-    if scipy.sparse.issparse(distance_matrix):
         if not isinstance(distance_matrix, scipy.sparse.csr_matrix):
-            # For now only return CSR format.
+            # Currently, we only return a sparse matrix in CSR format.
             distance_matrix = distance_matrix.tocsr()
 
-        # Sort_indices return immediately if indices are already sorted.
-        # If not sorted, the call can be costly (depending on nnz), but is better for
-        # later usage
+        # only for the sparse case we care about kmin:
+        if (kmin > 0 and not is_pdist) or (kmin > 1 and is_pdist):
+            # kmin == 1 and is_pdist does not need treatment because the diagonal is set.
+            distance_matrix = _ensure_kmin_nearest_neighbor(
+                X, Y, metric=metric, kmin=kmin, distance_matrix=distance_matrix,
+            )
+
+        # sort_indices returns immediately if indices are already sorted.
+        # If not sorted, the call could be costly (depending on nnz), but is better for
+        # follow-up handling.
         distance_matrix.sort_indices()
 
+        n_elements_stored = (
+            distance_matrix.nnz
+            + len(distance_matrix.indptr)
+            + len(distance_matrix.indices)
+        )
+        if n_elements_stored > np.product(distance_matrix.shape):
+            warnings.warn(
+                f"cut_off={cut_off} value does not lead to reduced memory requirements "
+                f"with sparse matrix. The sparse matrix stores {n_elements_stored} "
+                f"which exceeds a dense matrix by "
+                f"{n_elements_stored - np.product(distance_matrix.shape)} elements."
+            )
+
     return distance_matrix
 
 
