@@ -71,8 +71,8 @@ class TSCException(Exception):
 
 
 class _LocHandler(object):
-    """Required for overwriting the behavior of TSCDataFrame.loc.
-    iloc is currently not supported, see #61"""
+    """Required for overwriting the behavior of :meth:`TSCDataFrame.loc`.
+    `iloc` is currently not supported, see #61"""
 
     def __init__(self, tsc, method):
         # cast for the request back to a DataFrame
@@ -127,20 +127,17 @@ class TSCDataFrame(pd.DataFrame):
     """Data frame to represent collections of time series data.
 
     The class inherits from pandas' data structure :class:`pandas.DataFrame` and provides
-    additional methods to manipulate or analyse the data. The following main restrictions
-    are on the format:
+    additional methods to manipulate or analyse the time series collection. The following
+    main restrictions are made on a `pandas.DataFrame`:
 
     * two-dimensional index, where the first index indicates the time series ID (
       integer), and the second the time (non-negative numerical values)
-    * one dimensional columns for feature names
-    * neither in index nor in column names are duplicates allowed
+    * one-dimensional columns for feature names
+    * neither the index nor in column allows duplicates
 
-    Further restrictions are index and column names, however, they are internally
-    during initialization.
-
-    Please view the full Pandas `documentation
-    <https://pandas.pydata.org/pandas-docs/stable/index.html>`_ of inherited attributes
-    and methods and further algorithms that act on data frames.
+    Please visist the Pandas
+    `documentation <https://pandas.pydata.org/pandas-docs/stable/index.html>`_ for
+    inherited attributes, methods and other algorithms that act on data frames.
 
     .. note::
         Because Pandas provides a large variety of functionality of its data
@@ -149,8 +146,8 @@ class TSCDataFrame(pd.DataFrame):
         `<opening an issue <https://gitlab.com/datafold-dev/datafold/-/issues>`_.
 
     .. warning::
-        Currently, there is no `TSCSeries`, which results in some inconsistencies when
-        using `iloc`. Even if the result is still be a valid `TSCDataFrame` the type
+        Currently, there is no `TSCSeries`, which results in inconsistencies when
+        using `iloc`. Even if the result is still a valid `TSCDataFrame` the type
         changes to `pandas.DataFrame` or `pandas.Series`.
 
     Examples
@@ -185,19 +182,21 @@ class TSCDataFrame(pd.DataFrame):
 
     Parameters
     ----------
-    *args,**kwargs
-        all parameters are handled to superclass `pandas.DataFrame`
+    *args
+    **kwargs
+        All parameters (`*args` and `**kwargs`) are passed to superclass
+        ``pandas.DataFrame``.
 
     Attributes
     ----------
     tsc_id_idx_name
-        index name of first index to identify the time series
+        The index name of first index to select a time series.
 
     tsc_time_idx_name
-        index name of second index to select time
+        The index name of second index to select time.
     
     tsc_feature_col_name
-        column name to identify the feature
+        The name of feature axis (columns).
     """
 
     tsc_id_idx_name = "ID"  # name used in index of (unique) time series
@@ -220,7 +219,7 @@ class TSCDataFrame(pd.DataFrame):
     def from_tensor(
         cls,
         tensor: np.ndarray,
-        time_series_ids=None,
+        time_series_ids: Optional[np.ndarray] = None,
         columns: Optional[Union[pd.Index, list]] = None,
         time_values: Optional[np.ndarray] = None,
     ) -> "TSCDataFrame":
@@ -229,18 +228,19 @@ class TSCDataFrame(pd.DataFrame):
         Parameters
         ----------
         tensor
-            Time series data of shape `(n_timeseries, n_timesteps, n_feature,)`
+            The time series data of shape `(n_timeseries, n_timesteps, n_feature,)`.
 
         time_series_ids
-            ids of shape `(n_timeseries,)` to assign to respective time series.
+            The IDs of shape `(n_timeseries,)` to assign to respective time
+            series. Defaults to `(0,1,2,..., n_timeseries)`.
 
         columns
-            Feature names of with `(n_feature,)` elements. Defaults to `feature#`,
-            where `#` numbers the features consecutively.
+            The feature names of shape `(n_feature,)`. Defaults to
+            string `feature[0,1,2,..., n_feature]`.
 
         time_values
             Time values of the time series in the tensor. Defaults to
-            `numpy.arange(n_timesteps)`.
+            `(0,1,2, ..., n_timesteps)`.
 
         Returns
         -------
@@ -302,18 +302,18 @@ class TSCDataFrame(pd.DataFrame):
         Parameters
         ----------
         left_matrix
-            time series values at time "now"
+            Time series values at time "now".
 
         right_matrix
-            time series values at time "next"
+            Time series values at time "next".
 
         snapshot_orientation
-            Indicate whether the snapshots (states) are in rows ("row") or columns (
-            "col").
+            Indicate whether the snapshots (states) are in rows ("row") or columns
+            ("col").
             
         columns
-            Feature names of with `(n_feature,)` elements. Defaults to `feature#`,
-            where `#` numbers the features consecutively.
+            Feature names of shape `(n_feature,)`. Defaults to
+            `feature[0,1,2,..., n_feature]`.
 
         Returns
         -------
@@ -370,18 +370,18 @@ class TSCDataFrame(pd.DataFrame):
         Parameters
         ----------
         indices_from
-            Existing object to copy index and/or columns.
+            Existing object to copy index and/or columns from.
 
         values
-            values for new time series collection
+            Values for new time series collection.
 
         except_index
             Index for new time series collection (only copy columns). Should not be
-            given together with `except_columns`.
+            set together with `except_columns`.
 
         except_columns
             Columns for this time series collection (only copy index). Should not be
-            given together with `except_index`.
+            set together with `except_index`.
 
         Returns
         -------
@@ -427,7 +427,7 @@ class TSCDataFrame(pd.DataFrame):
             Time series data with time values in index and features in columns.
 
         ts_id
-            ID of time series
+            ID of initial time series.
 
         Returns
         -------
@@ -454,13 +454,12 @@ class TSCDataFrame(pd.DataFrame):
 
     @classmethod
     def from_frame_list(cls, frame_list: List[pd.DataFrame]) -> "TSCDataFrame":
-        """Initialize a time series collection from a list of `pandas.DataFrames`
-        representing time series.
+        """Initialize a time series collection from a list of time series.
 
         Parameters
         ----------
         frame_list
-            data frames with `index=time_values` and `columns=feature_names`
+            Data frames with :code:`index=time_values` and :code:`columns=feature_names`.
 
         Returns
         -------
@@ -492,11 +491,11 @@ class TSCDataFrame(pd.DataFrame):
         Parameters
         ----------
         filepath
-            file path
+            The file path to the csv file.
 
         **kwargs
-            key word arguments handled to
-            `pandas.read_csv <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_csv.html>`_ # noqa
+            keyword arguments handled to
+            `pandas.read_csv <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_csv.html>`_
 
         Returns
         -------
@@ -612,19 +611,19 @@ class TSCDataFrame(pd.DataFrame):
 
     @property
     def n_timeseries(self) -> int:
-        """Number of time series in collection.
+        """Number of time series in the collection.
         """
         return len(self.ids)
 
     @property
     def n_features(self) -> int:
-        """Number of features in collection.
+        """Number of features in the collection.
         """
         return self.shape[1]
 
     @property
     def ids(self) -> pd.Index:
-        """Present time series IDs .
+        """The time series IDs in the collection.
         """
         # update index by removing potentially unused levels
         self.index = self.index.remove_unused_levels()
@@ -632,10 +631,10 @@ class TSCDataFrame(pd.DataFrame):
 
     @property
     def delta_time(self) -> Union[pd.Series, float]:
-        """Time value frequency.
+        """Time sampling frequency.
 
-        Collects for each time series the time delta, irregular frequencies are marked
-        with `nan`. If all time series are consistent (all have same time delta,
+        Collects for each time series the time delta. Irregular frequencies are marked
+        with `nan`. If all time series are consistent (i.e., all have same time delta,
         including `nan`), then a single float is returned.
 
         Returns
@@ -699,7 +698,7 @@ class TSCDataFrame(pd.DataFrame):
 
     @property
     def n_timesteps(self) -> Union[int, pd.Series]:
-        """Number of time values
+        """Number of time steps per time series.
 
         Collects for each time series the number of time steps. If all time series are
         consistent (all have same time steps), then a single float is returned.
@@ -726,13 +725,13 @@ class TSCDataFrame(pd.DataFrame):
         
         The following types are returned:
 
-        * `TSCDataFrame` if the slice is still valid
-        * `pandas.DataFrame` fallback if slice is not a valid TSCDataFrame anymore and
-           contains multiple features
-        * `pandas.Series` if slice is not a valid TSCDataFrame anymore and contains a
-           single feature
+        * ``TSCDataFrame`` if the slice is still valid
+        * ``pandas.DataFrame`` fallback if slice is not a valid ``TSCDataFrame`` anymore\
+         and contains multiple features
+        * ``pandas.Series`` if slice is not a valid ``TSCDataFrame`` anymore and contains\
+         a single feature.
 
-        All other rules of `.loc` slicing rules apply.
+        All other rules of `.loc` slicing apply.
 
         Returns
         -------
@@ -747,19 +746,20 @@ class TSCDataFrame(pd.DataFrame):
     def xs(
         self, key, axis=0, level=None, drop_level: bool = True
     ) -> Union[pd.DataFrame, pd.Series]:
-        """Overwrites cross section to provide fall back type in case the result
+        """Overwrites cross section to provide fall back solution in case the result
         is not a valid ``TSCDataFrame`` anymore.
 
         Parameters
         ----------
         args
             see docu
-            `DataFrame.xs <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.xs.html#pandas.DataFrame.xs>`_ # noqa
+            `DataFrame.xs <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.xs.html#pandas.DataFrame.xs>`_
 
         Returns
         -------
-        Union[TSCDataFrame, pd.DataFrame, pd.Series]
-            cross section, `TSCDataFrame` if still valid, else the direct pandas type
+        Union[TSCDataFrame, pandas.DataFrame, pandas.Series]
+            Cross section of ``TSCDataFrame``. The type is determined by the resulting
+            slice.
         """
 
         _internal_df = pd.DataFrame(self)
@@ -792,12 +792,12 @@ class TSCDataFrame(pd.DataFrame):
         """Generator over time series (id, pandas.DataFrame).
 
         .. note::
-            Each time series is a `pandas.DataFrame` (not a TSCDataFrame).
+            Each time series is a ``pandas.DataFrame`` (not a ``TSCDataFrame``).
 
         Yields
         ------
         Tuple[int, pandas.DataFrame]
-            time series id and corresponding time series
+            Time series ID and corresponding time series.
         """
         for i, ts in self.groupby(level=self.tsc_id_idx_name):
             # cast single time series back to DataFrame
@@ -810,7 +810,7 @@ class TSCDataFrame(pd.DataFrame):
         return len(np.unique(self.n_timesteps)) == 1
 
     def is_const_delta_time(self) -> bool:
-        """Indicates if all time series in the collection have the same delta time.
+        """Indicates if all time series in the collection have the same time delta.
         """
 
         # If dt is a Series it means it shows "dt per ID" (because it is not constant).
@@ -849,6 +849,7 @@ class TSCDataFrame(pd.DataFrame):
         """Indicates if the time values are normalized.
 
         A normalized time has the following properties:
+
             * the first time record is zero (not necessarily for all time series), and
             * `delta_time` is constant 1 for all time series
         """
@@ -868,7 +869,7 @@ class TSCDataFrame(pd.DataFrame):
 
         Parameters
         ----------
-        df: pd.DataFrame
+        df
             new time series, with same features (column names)
         ts_id
             Dedicated ID of new time series (must not be present in current collection).
@@ -961,7 +962,7 @@ class TSCDataFrame(pd.DataFrame):
         )
 
     def feature_to_array(self, feature: str) -> np.ndarray:
-        """Turns a single feature column into data array.
+        """Turns a single feature column into a matrix.
 
         Parameters
         ----------
@@ -971,12 +972,12 @@ class TSCDataFrame(pd.DataFrame):
         Returns
         -------
         numpy.ndarray
-            feature values with shape `(n_timeseries, n_timesteps)`
+            feature values of shape `(n_timeseries, n_timesteps)`
 
         Raises
         ------
         TSCException
-            if time series have not identical time values
+            If time series in the collection have not identical time values.
 
         """
 
@@ -990,7 +991,7 @@ class TSCDataFrame(pd.DataFrame):
     def select_time_values(
         self, time_values: Union[int, float, np.ndarray]
     ) -> Union[pd.DataFrame, "TSCDataFrame"]:
-        """Select time values each time series (if available).
+        """Select time values of all time series with a matching time value.
 
         Parameters
         ----------
@@ -1001,31 +1002,31 @@ class TSCDataFrame(pd.DataFrame):
         -------
         Union[pd.DataFrame, "TSCDataFrame"]
             If sliced data frame is not a valid `TSCDataFrame`, the fallback is to
-            a `DataFrame`
+            a ``pandas.DataFrame``.
 
         """
         idx = pd.IndexSlice
         return self.loc[idx[:, time_values], :]
 
     def initial_states(self, n_samples: int = 1) -> Union[pd.DataFrame, "TSCDataFrame"]:
-        """Get initial states of each time series in the collection.
+        """Get initial state of each time series in the collection.
 
         Parameters
         ----------
 
         n_samples
-            number of samples required for initial state
+            Number of samples required for an initial state.
 
         Returns
         -------
         Union[pd.DataFrame, TSCDataFrame]
-            initial states with shape `(n_samples, n_features)`
+            Initial states of shape `(n_samples * n_timeseries, n_features)`.
 
         Raises
         ------
         TSCException
-            if there is a time series with less than the required number of samples
-
+            If there is a time series in the collection that has less time values
+            than than the required number of samples.
         """
 
         if not is_integer(n_samples) or n_samples < 1:
@@ -1054,12 +1055,12 @@ class TSCDataFrame(pd.DataFrame):
         Returns
         -------
         Union[pd.DataFrame, TSCDataFrame]
-            final states with shape `(n_samples, n_features)`
+            The final states of shape `(n_samples, n_features)`.
 
         Raises
         ------
         TSCException
-            if there is a time series with less than the required number of samples
+            If there is a time series with less than the required number of samples.
 
         """
         if not is_integer(n_samples) or n_samples < 1:
@@ -1067,7 +1068,7 @@ class TSCDataFrame(pd.DataFrame):
 
         # only larger than 2 because by definition each time series
         # has a minimum of 2 time samples
-        if n_samples > 2 and not np.asarray(self.n_timesteps <= n_samples).all():
+        if n_samples > 2 and not np.asarray(n_samples <= self.n_timesteps).all():
             raise TSCException.not_required_n_timesteps(
                 required_length=n_samples, actual_length=self.n_timesteps
             )
@@ -1088,9 +1089,11 @@ class TSCDataFrame(pd.DataFrame):
             Key word arguments handled to each time series
             `pandas.DataFrame.plot() <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.plot.html?highlight=plot#pandas.DataFrame.plot>`_
             call.
+
         Returns
         -------
-
+        matplotlib object
+            axes handle
         """
         ax = kwargs.pop("ax", None)
         legend = kwargs.pop("legend", True)
@@ -1114,14 +1117,18 @@ class TSCDataFrame(pd.DataFrame):
 
 
 class InitialCondition(object):
-    """Collection of helper functions to create and validate initial conditions for
-    time series predictions.
+    """Helper functions to create and validate initial conditions for time series
+    predictions.
 
-    Initial conditions are required models that allow time series predictions,
-    for example in :py:meth:`EDMD.predict`. An initial condition can be either single
-    states (e.g. a vector at time zero), or a time series itself. This is the case,
-    if the model transforms the time series internally and requires multiple time
-    values for an embedding (e.g. ``TSCTakensEmbedding``).
+    Initial conditions are described with ``pandas.DataFrame`` objects
+    (including :class:`.TSCDataFrame`). However, a user can also provide a
+    :class:`numpy.ndarray`, but there are less checks for input validation possible.
+
+    In general, initial conditions are required in models that train on time series
+    data, see for example in :py:meth:`EDMD.predict`. An initial condition can consist of
+    single states (e.g. a vector at time zero), or a time series itself. The latter is
+    the case if model transformations require multiple time values to define the
+    transformed state.
     """
 
     @classmethod
@@ -1133,11 +1140,11 @@ class InitialCondition(object):
         Parameters
         ----------
         X
-            initial conditions with shape `(n_ic, n_features)`
+            Initial condition of shape `(n_ic, n_features)`.
 
         columns
-            feature names in model during fit (they can be accessed with
-            :code:`model_obj.features_in_[1]`
+            Feature names in model during fit (they can be accessed with
+            :code:`model_obj.features_in_[1]`.
 
         Returns
         -------
@@ -1166,21 +1173,21 @@ class InitialCondition(object):
 
     @classmethod
     def from_tsc(cls, X: TSCDataFrame, n_samples_ic: int = 1) -> pd.DataFrame:
-        """Collects all initial states from a `TSCDataFrame`.
+        """Collects all initial states from a ``TSCDataFrame``.
         
         .. note::
+
             The time values of the time series' initial states are ignored for single
             states. For time series with multiple timesteps, the initial condition is
             invalid if the time values are not identical between initial states.
 
-        
         Parameters
         ----------
         X
-            extract initial states from
+            Object to extract initial states from.
 
         n_samples_ic
-            number of time steps per initial condition
+            Number of time steps per initial condition.
 
         Returns
         -------
@@ -1204,18 +1211,18 @@ class InitialCondition(object):
     def iter_reconstruct_ic(
         cls, X: TSCDataFrame, n_samples_ic: int = 1
     ) -> Generator[Tuple[pd.DataFrame, np.ndarray], None, None]:
-        """Extract and iterates over initial conditions and respective time values of
-        time series grouping series with identical time values.
+        """Extract and iterate over initial conditions with time series groups of that
+        have identical time values.
 
-        This iterator is especially useful to reconstruct sets of time series.
+        This iterator is useful to reconstruct time series.
         
         Parameters
         ----------
         X
-            extract initial states from
+            Object to extract initial states from.
 
         n_samples_ic
-            number of time steps per initial condition
+            The number of time steps per initial condition.
 
         Returns
         -------
@@ -1321,7 +1328,7 @@ class InitialCondition(object):
         Parameters
         ----------
         X_ic
-            initial condition to validate
+            The initial condition to validate.
 
         Returns
         -------
@@ -1345,10 +1352,10 @@ class InitialCondition(object):
 
 
 def allocate_time_series_tensor(n_time_series, n_timesteps, n_feature):
-    """Allocate time series tensor that complies with
-    :py:meth:`TSCDataFrame.from_tensor()`
+    """Allocate a time series tensor that complies with
+    :py:meth:`TSCDataFrame.from_tensor()`.
 
-    Allocated three-dimensional numpy.ndarray is C-aligned and can be accessed with
+    Allocated three-dimensional ``numpy.ndarray`` is C-aligned and can be accessed with
 
     .. code::
 
@@ -1357,13 +1364,13 @@ def allocate_time_series_tensor(n_time_series, n_timesteps, n_feature):
     Parameters
     ----------
     n_time_series
-        Number of time series.
+        The number of time series.
 
     n_timesteps
-        Number of time values for all time series.
+        The number of time values of a single time series.
 
     n_feature
-        Number of features.
+        The number of features for each time series.
 
     Returns
     -------
